# -*- coding: utf-8 -*-
from abjad.tools import systemtools
from abjad.tools.durationtools.Duration import Duration


class Offset(Duration):
    '''Offset.

    ..  container:: example

        **Example 1.** Initializes from integer numerator:

        ::

            >>> Offset(3)
            Offset(3, 1)

    ..  container:: example

        **Example 2.** Initializes from integer numerator and denominator:

        ::

            >>> Offset(3, 16)
            Offset(3, 16)

    ..  container:: example

        **Example 3.** Initializes from integer-equivalent numeric numerator:

        ::

            >>> Offset(3.0)
            Offset(3, 1)

    ..  container:: example

        **Example 4.** Initializes from integer-equivalent numeric numerator
        and denominator:

        ::

            >>> Offset(3.0, 16)
            Offset(3, 16)

    ..  container:: example

        **Example 5.** Initializes from integer-equivalent singleton:

        ::

            >>> Offset((3,))
            Offset(3, 1)

    ..  container:: example

        **Example 6.** Initializes from integer-equivalent pair:

        ::

            >>> Offset((3, 16))
            Offset(3, 16)

    ..  container:: example

        **Example 7.** Initializes from duration:

        ::

            >>> Offset(Duration(3, 16))
            Offset(3, 16)

    ..  container:: example

        **Example 8.** Initializes from other offset:

        ::

            >>> Offset(Offset(3, 16))
            Offset(3, 16)

    ..  container:: example

        **Example 9.** Initializes from other offset with grace displacement:

        ::

            >>> offset = Offset((3, 16), grace_displacement=(-1, 16))
            >>> Offset(offset)
            Offset(
                (3, 16),
                grace_displacement=Duration(-1, 16)
                )

    ..  container:: example

        **Example 10.** Intializes from fraction:

        ::

            >>> Offset(Fraction(3, 16))
            Offset(3, 16)

    ..  container:: example

        **Example 11.** Initializes from solidus string:

        ::

            >>> Offset('3/16')
            Offset(3, 16)

    ..  container:: example

        **Example 12.** Initializes from nonreduced fraction:

        ::

            >>> Offset(mathtools.NonreducedFraction(6, 32))
            Offset(3, 16)

    ..  container:: example

        **Example 13.** Offsets inherit from built-in fraction:

        ::

            >>> isinstance(Offset(3, 16), Fraction)
            True

    ..  container:: example

        **Example 14.** Offsets are numeric:

        ::

            >>> import numbers

        ::

            >>> isinstance(Offset(3, 16), numbers.Number)
            True

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_grace_displacement',
        )

    ### CONSTRUCTOR ###

    def __new__(class_, *arguments, **keywords):
        grace_displacement = None
        for arg in arguments:
            if hasattr(arg, 'grace_displacement'):
                grace_displacement = getattr(arg, 'grace_displacement')
                break
        grace_displacement = grace_displacement or keywords.get(
            'grace_displacement')
        if grace_displacement is not None:
            grace_displacement = Duration(grace_displacement)
        grace_displacement = grace_displacement or None
        if len(arguments) == 1 and isinstance(arguments[0], Duration):
            arguments = arguments[0].pair
        self = Duration.__new__(class_, *arguments)
        self._grace_displacement = grace_displacement
        return self

    ### SPECIAL METHODS ###

    def __copy__(self, *arguments):
        r'''Copies offset.

        ::

            >>> import copy

        ..  container:: example

            **Example 1.** Copies offset with grace displacement:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 16))
                >>> offset_2 = copy.copy(offset_1)

            ::

                >>> offset_1
                Offset(
                    (1, 4),
                    grace_displacement=Duration(-1, 16)
                    )

            ::

                >>> offset_2
                Offset(
                    (1, 4),
                    grace_displacement=Duration(-1, 16)
                    )

            ::

                >>> offset_1 == offset_2
                True

            ::

                >>> offset_1 is offset_2
                False

        Returns new offset.
        '''
        arguments = self.__getnewargs__()
        return type(self)(
            *arguments,
            grace_displacement=self.grace_displacement
            )

    def __deepcopy__(self, *arguments):
        r'''Deep copies offset.

        ::

            >>> import copy

        ..  container:: example

            **Example 1.** Copies offset with grace displacement:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 16))
                >>> offset_2 = copy.deepcopy(offset_1)

            ::

                >>> offset_1
                Offset(
                    (1, 4),
                    grace_displacement=Duration(-1, 16)
                    )

            ::

                >>> offset_2
                Offset(
                    (1, 4),
                    grace_displacement=Duration(-1, 16)
                    )

            ::

                >>> offset_1 == offset_2
                True

            ::

                >>> offset_1 is offset_2
                False

        Returns new offset.
        '''
        return self.__copy__(*arguments)

    def __eq__(self, arg):
        r'''Is true when offset equals `arg`.
        Otherwise false.

        ..  container:: example

            **Example 1.** With equal numerators, denominators and grace
            displacements:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 16))
                >>> offset_2 = Offset((1, 4), grace_displacement=(-1, 16))

            ::

                >>> offset_1 == offset_1
                True
                >>> offset_1 == offset_2
                True
                >>> offset_2 == offset_1
                True
                >>> offset_2 == offset_2
                True

        ..  container:: example

            **Example 2.** With equal numerators and denominators but differing
            grace displacements:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 8))
                >>> offset_2 = Offset((1, 4), grace_displacement=(-1, 16))

            ::

                >>> offset_1 == offset_1
                True
                >>> offset_1 == offset_2
                False
                >>> offset_2 == offset_1
                False
                >>> offset_2 == offset_2
                True

        ..  container:: example

            **Example 3.** With differing numerators and denominators. Ignores
            grace displacements:

            ::

                >>> offset_1 = Offset((1, 4))
                >>> offset_2 = Offset((1, 2), grace_displacement=(-99))

            ::

                >>> offset_1 == offset_1
                True
                >>> offset_1 == offset_2
                False
                >>> offset_2 == offset_1
                False
                >>> offset_2 == offset_2
                True

        Returns true or false.
        '''
        if isinstance(arg, type(self)) and self.pair == arg.pair:
            return self._get_grace_displacement() == \
                arg._get_grace_displacement()
        return super(Offset, self).__eq__(arg)

    def __ge__(self, arg):
        r'''Is true when offset is greater than or equal to `arg`.
        Otherwise false.

        ..  container:: example

            **Example 1.** With equal numerators, denominators and grace
            displacements:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 16))
                >>> offset_2 = Offset((1, 4), grace_displacement=(-1, 16))

            ::

                >>> offset_1 >= offset_1
                True
                >>> offset_1 >= offset_2
                True
                >>> offset_2 >= offset_1
                True
                >>> offset_2 >= offset_2
                True

        ..  container:: example

            **Example 2.** With equal numerators and denominators but differing
            grace displacements:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 8))
                >>> offset_2 = Offset((1, 4), grace_displacement=(-1, 16))

            ::

                >>> offset_1 >= offset_1
                True
                >>> offset_1 >= offset_2
                False
                >>> offset_2 >= offset_1
                True
                >>> offset_2 >= offset_2
                True

        ..  container:: example

            **Example 3.** With differing numerators and denominators. Ignores
            grace displacements:

            ::

                >>> offset_1 = Offset((1, 4))
                >>> offset_2 = Offset((1, 2), grace_displacement=(-99))

            ::

                >>> offset_1 >= offset_1
                True
                >>> offset_1 >= offset_2
                False
                >>> offset_2 >= offset_1
                True
                >>> offset_2 >= offset_2
                True

        Returns true or false.
        '''
        if isinstance(arg, type(self)) and self.pair == arg.pair:
            return self._get_grace_displacement() >= \
                arg._get_grace_displacement()
        return super(Offset, self).__ge__(arg)

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return self.pair

    def __gt__(self, arg):
        r'''Is true when offset is greater than `arg`.
        Otherwise false.

        ..  container:: example

            **Example 1.** With equal numerators, denominators and grace
            displacements:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 16))
                >>> offset_2 = Offset((1, 4), grace_displacement=(-1, 16))

            ::

                >>> offset_1 > offset_1
                False
                >>> offset_1 > offset_2
                False
                >>> offset_2 > offset_1
                False
                >>> offset_2 > offset_2
                False

        ..  container:: example

            **Example 2.** With equal numerators and denominators but differing
            grace displacements:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 8))
                >>> offset_2 = Offset((1, 4), grace_displacement=(-1, 16))

            ::

                >>> offset_1 > offset_1
                False
                >>> offset_1 > offset_2
                False
                >>> offset_2 > offset_1
                True
                >>> offset_2 > offset_2
                False

        ..  container:: example

            **Example 3.** With differing numerators and denominators. Ignores
            grace displacements:

            ::

                >>> offset_1 = Offset((1, 4))
                >>> offset_2 = Offset((1, 2), grace_displacement=(-99))

            ::

                >>> offset_1 > offset_1
                False
                >>> offset_1 > offset_2
                False
                >>> offset_2 > offset_1
                True
                >>> offset_2 > offset_2
                False

        Returns true or false.
        '''
        if isinstance(arg, type(self)) and self.pair == arg.pair:
            return self._get_grace_displacement() > \
                arg._get_grace_displacement()
        return Duration.__gt__(self, arg)

    def __hash__(self):
        r'''Hashes duration.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return hash((
            type(self),
            self.numerator,
            self.denominator,
            self.grace_displacement,
            ))

    def __le__(self, arg):
        r'''Is true when offset is less than or equal to `arg`.
        Otherwise false.

        ..  container:: example

            **Example 1.** With equal numerators, denominators and grace
            displacements:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 16))
                >>> offset_2 = Offset((1, 4), grace_displacement=(-1, 16))

            ::

                >>> offset_1 <= offset_1
                True
                >>> offset_1 <= offset_2
                True
                >>> offset_2 <= offset_1
                True
                >>> offset_2 <= offset_2
                True

        ..  container:: example

            **Example 2.** With equal numerators and denominators but differing
            grace displacements:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 8))
                >>> offset_2 = Offset((1, 4), grace_displacement=(-1, 16))

            ::

                >>> offset_1 <= offset_1
                True
                >>> offset_1 <= offset_2
                True
                >>> offset_2 <= offset_1
                False
                >>> offset_2 <= offset_2
                True

        ..  container:: example

            **Example 3.** With differing numerators and denominators. Ignores
            grace displacements:

            ::

                >>> offset_1 = Offset((1, 4))
                >>> offset_2 = Offset((1, 2), grace_displacement=(-99))

            ::

                >>> offset_1 <= offset_1
                True
                >>> offset_1 <= offset_2
                True
                >>> offset_2 <= offset_1
                False
                >>> offset_2 <= offset_2
                True

        Returns true or false.
        '''
        if isinstance(arg, type(self)) and self.pair == arg.pair:
            return self._get_grace_displacement() <= \
                arg._get_grace_displacement()
        return super(Offset, self).__le__(arg)

    def __lt__(self, arg):
        r'''Is true when offset is less than `arg`.
        Otherwise false.

        ..  container:: example

            **Example 1.** With equal numerators, denominators and grace
            displacements:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 16))
                >>> offset_2 = Offset((1, 4), grace_displacement=(-1, 16))

            ::

                >>> offset_1 < offset_1
                False
                >>> offset_1 < offset_2
                False
                >>> offset_2 < offset_1
                False
                >>> offset_2 < offset_2
                False

        ..  container:: example

            **Example 2.** With equal numerators and denominators but differing
            nonzero grace displacements:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 8))
                >>> offset_2 = Offset((1, 4), grace_displacement=(-1, 16))

            ::

                >>> offset_1 < offset_1
                False
                >>> offset_1 < offset_2
                True
                >>> offset_2 < offset_1
                False
                >>> offset_2 < offset_2
                False

        ..  container:: example

            **Example 3.** With equal numerators and denominators but differing
            zero-valued grace displacements:

            ::

                >>> offset_1 = Offset((1, 4), grace_displacement=(-1, 8))
                >>> offset_2 = Offset((1, 4))

            ::

                >>> offset_1 < offset_1
                False
                >>> offset_1 < offset_2
                True
                >>> offset_2 < offset_1
                False
                >>> offset_2 < offset_2
                False

        ..  container:: example

            **Example 4.** With differing numerators and denominators. Ignores
            grace displacements:

            ::

                >>> offset_1 = Offset((1, 4))
                >>> offset_2 = Offset((1, 2), grace_displacement=(-99))

            ::

                >>> offset_1 < offset_1
                False
                >>> offset_1 < offset_2
                True
                >>> offset_2 < offset_1
                False
                >>> offset_2 < offset_2
                False

        Returns true or false.
        '''
        if isinstance(arg, type(self)) and self.pair == arg.pair:
            return self._get_grace_displacement() < \
                arg._get_grace_displacement()
        return super(Offset, self).__lt__(arg)

    def __repr__(self):
        r'''Gets interpreter representation of offset.

        ..  container:: example

            **Example 1.** Gets interpreter representation of offset without
            grace displacement:

            ::

                >>> Offset(1, 4)
                Offset(1, 4)

        ..  container:: example

            **Example 2.** Gets interpreter representation of offset with
            grace displacement:

            ::

                >>> Offset(1, 4, grace_displacement=(-1, 16))
                Offset(
                    (1, 4),
                    grace_displacement=Duration(-1, 16)
                    )

        '''
        return super(Offset, self).__repr__()

    def __sub__(self, argument):
        '''Offset taken from offset returns duration:

        ::

            >>> durationtools.Offset(2) - durationtools.Offset(1, 2)
            Duration(3, 2)

        Duration taken from offset returns another offset:

        ::

            >>> durationtools.Offset(2) - durationtools.Duration(1, 2)
            Offset(3, 2)

        Coerce `argument` to offset when `argument` is neither offset nor duration:

        ::

            >>> durationtools.Offset(2) - Fraction(1, 2)
            Duration(3, 2)

        Returns duration or offset.
        '''
        if isinstance(argument, type(self)):
            return Duration(super(Offset, self).__sub__(argument))
        elif isinstance(argument, Duration):
            return super(Offset, self).__sub__(argument)
        else:
            argument = type(self)(argument)
            return self - argument

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        is_indented = False
        names = []
        values = [self.numerator, self.denominator]
        if self._get_grace_displacement():
            is_indented = True
            names = ['grace_displacement']
            values = [(self.numerator, self.denominator)]
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=is_indented,
            storage_format_args_values=values,
            storage_format_is_indented=is_indented,
            storage_format_kwargs_names=names,
            )

    def _get_grace_displacement(self):
        from abjad.tools import durationtools
        if self.grace_displacement is None:
            return durationtools.Duration(0)
        return self.grace_displacement

    ### PUBLIC PROPERTIES ###

    @property
    def grace_displacement(self):
        r'''Gets grace displacement.

        ..  container:: example

            **Example 1.** Gets grace displacement equal to none:

            ::

                >>> offset = Offset(1, 4)
                >>> offset.grace_displacement is None
                True

        ..  container:: example

            **Example 2.** Gets grace displacement equal to a negative
            sixteenth:

            ::

                >>> offset = Offset(1, 4, grace_displacement=(-1, 16))
                >>> offset.grace_displacement
                Duration(-1, 16)

        ..  container:: example

            **Example 3.** Stores zero-valued grace displacement as none:

            ::

                >>> offset = Offset(1, 4, grace_displacement=0)
                >>> offset.grace_displacement is None
                True

            ::

                >>> offset
                Offset(1, 4)

        Defaults to none.

        Set to duration or none.

        Returns duration or none.
        '''
        return self._grace_displacement



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Beam__fuse_by_reference_01():
    r'''Fuse beams.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    left_beam = Beam()
    attach(left_beam, staff[:2])
    right_beam = Beam()
    attach(right_beam, staff[2:4])
    left_beam._fuse_by_reference(right_beam)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
            g'8
            a'8
            b'8
            c''8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-


def persist(client):
    r'''Makes persistence agent.

    ..  container:: example

        Persists staff as LilyPond file:

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                e'4
                d'4
                f'4
            }

        ::

            >>> persist(staff).as_ly() # doctest: +SKIP

    ..  container:: example

        Returns persistence agent:

        ::

            >>> persist(staff)
            PersistenceAgent(client=Staff("c'4 e'4 d'4 f'4"))

    '''
    from abjad.tools import agenttools
    return agenttools.PersistenceAgent(client)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_repeated_notes_from_time_signatures_01():
    r'''Make repeated notes from list of integer pairs.
    '''

    notes = scoretools.make_repeated_notes_from_time_signatures(
        [(2, 8), (3, 32)], pitch = "d''")
    assert len(notes) == 2

    notes = Sequence(notes).flatten()
    staff = Staff(notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            d''8
            d''8
            d''32
            d''32
            d''32
        }
        '''
        )
    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.demos.ferneyhough.make_nested_tuplet import make_nested_tuplet


def make_row_of_nested_tuplets(
    tuplet_duration,
    outer_tuplet_proportions,
    column_count,
    ):
    r'''Makes row of nested tuplets.
    '''

    assert 0 < column_count
    row_of_nested_tuplets = []
    for n in range(column_count):
        inner_tuplet_subdivision_count = n + 1
        nested_tuplet = make_nested_tuplet(
            tuplet_duration,
            outer_tuplet_proportions,
            inner_tuplet_subdivision_count,
            )
        row_of_nested_tuplets.append(nested_tuplet)
    return row_of_nested_tuplets



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedPitchClass_invert_01():

    assert pitchtools.NumberedPitchClass(0).invert() == pitchtools.NumberedPitchClass(0)
    assert pitchtools.NumberedPitchClass(1).invert() == pitchtools.NumberedPitchClass(11)
    assert pitchtools.NumberedPitchClass(2).invert() == pitchtools.NumberedPitchClass(10)
    assert pitchtools.NumberedPitchClass(3).invert() == pitchtools.NumberedPitchClass(9)
    assert pitchtools.NumberedPitchClass(4).invert() == pitchtools.NumberedPitchClass(8)
    assert pitchtools.NumberedPitchClass(5).invert() == pitchtools.NumberedPitchClass(7)
    assert pitchtools.NumberedPitchClass(6).invert() == pitchtools.NumberedPitchClass(6)
    assert pitchtools.NumberedPitchClass(7).invert() == pitchtools.NumberedPitchClass(5)
    assert pitchtools.NumberedPitchClass(8).invert() == pitchtools.NumberedPitchClass(4)
    assert pitchtools.NumberedPitchClass(9).invert() == pitchtools.NumberedPitchClass(3)
    assert pitchtools.NumberedPitchClass(10).invert() == pitchtools.NumberedPitchClass(2)
    assert pitchtools.NumberedPitchClass(11).invert() == pitchtools.NumberedPitchClass(1)



#! /usr/bin/env python
'''
Validate ### ... ### header order and section contents in classes.

Could be extended to check for other similar errors as needed.
'''

import os
import sys
from abjad.tools.commandlinetools.CommandlineScript import CommandlineScript


class CheckClassSections(CommandlineScript):
    r'''Checks the order and contents of class sections in a path.

    .. shell::

        ajv check-class-sections --help

    '''

    ### CLASS VARIABLES ###

    alias = 'check-class-sections'
    short_description = (
        'Check the order and contents of class sections in a path or file.'
        )
    long_description = ('''
Finds and lists errors in class section order,
as well as cases where methods and properties appear
under incorrect section headers

If no `path` is given, the current directory is searched.

Checks that where they appear in classes, the following
comment headers appear in the given order:

        ### CLASS VARIABLES ###
        ### CONSTRUCTOR ###
        ### INITIALIZER ###
        ### SPECIAL METHODS ###
        ### PRIVATE METHODS ###
        ### PUBLIC METHODS ###
        ### PRIVATE PROPERTIES ###
        ### PUBLIC PROPERTIES ###

Additionally, this finds cases where methods appear
under PROPERTIES sections, and vice-versa.'''
        )

    ### PRIVATE METHODS ###

    def _setup_argument_parser(self, parser):
        parser.add_argument(
            'path',
            default=os.getcwd(),
            help='file or path to check',
            nargs='?',
            )

    def _process_args(self, arguments):
        failed_files = 0
        checked_files = 0
        line_divider = '=' * 79
        file_list = []
        if os.path.isdir(arguments.path):
            relative_path_name = os.path.relpath(arguments.path)
            print('Recursively scanning {} for errors...'.format(
                'current working directory' if relative_path_name == '.'
                else relative_path_name
                )
            )
            for path, dirs, files in os.walk(arguments.path):
                for f in sorted(files):
                    file_list.append(os.path.abspath(os.path.join(path, f)))
        elif os.path.isfile(arguments.path):
            print('Scanning {} for errors...'.format(arguments.path))
            file_list.append(arguments.path)
        else:
            print('{} is not a valid directory or file'.format(arguments.path))
            sys.exit(1)
        for test_file in file_list:
            # Skip links and non-.py files
            if (os.path.islink(test_file) or
                (not test_file.endswith('.py'))
                ):
                continue
            errors = self._check_class_sections(test_file)
            checked_files += 1
            if any(e for e in errors.values()):
                failed_files += 1
                print('Errors in {}:'.format(test_file))
                for error in sorted(errors.items(), key=lambda e: e[0]):
                    if not error[1]:
                        continue
                    print('Lines {}: {}'.format(error[1], error[0]))
                print(line_divider)
        # Done
        print(
            '{} total files checked.\n'
            '{} passed.\n'
            '{} failed.\n'.format(
                checked_files,
                checked_files - failed_files,
                failed_files
                )
            )
        if failed_files:
            sys.exit(1)
        else:
            sys.exit(0)

    @staticmethod
    def _check_class_sections(file_path):
        r'''Check a file for a small subset of code-quality errors.

        Returns a dict of form {'ERROR TYPE': [line_numbers]}.
        '''
        headers = [
            '[no header]',
            '    ### CLASS VARIABLES ###\n',
            '    ### CONSTRUCTOR ###\n',
            '    ### INITIALIZER ###\n',
            '    ### SPECIAL METHODS ###\n',
            '    ### PRIVATE METHODS ###\n',
            '    ### PUBLIC METHODS ###\n',
            '    ### PRIVATE PROPERTIES ###\n',
            '    ### PUBLIC PROPERTIES ###\n'
            ]
        non_property_decorators = [
            '    @staticmethod',
            '    @classmethod',
            '    @abc.abstractmethod',
            '    @lex.TOKEN(',
            ]
        current_header = ''
        current_header_index = 0
        # Dict of error classes with a list of violating line numbers
        errors = {
            'BAD HEADER ORDER': [],
            'PROPERTY IN METHODS SECTION': [],
            'METHOD IN PROPERTIES SECTION': []
            }
        with open(file_path, 'r') as f:
            lines = f.readlines()

        for line_num in range(0, len(lines)):
            if lines[line_num].startswith('class '):
                # Reset header after running into a new class
                current_header = '[no header]'
                current_header_index = 0
            elif lines[line_num].startswith('    ### '):
                # This looks like a header - compare against known headers
                for header_index, header in enumerate(headers):
                    if lines[line_num] == header:
                        # We recognize this header: check if its order
                        if header_index < current_header_index:
                            errors['BAD HEADER ORDER'].append(line_num)
                        current_header_index = header_index
                        current_header = header
                        break
                else:
                    # Even if we don't recognize this header, we can still
                    # use it when detecting certain out-of-place
                    # methods / properties, so keep track of it
                    current_header_index = 0
                    current_header = lines[line_num]
            # Check that other contents make sense under the last-seen header
            elif 'METHODS' in current_header:
                if (
                    lines[line_num].startswith('    @') and not (
                    # Whitelist known non-property decorators
                    any(lines[line_num].startswith(dec)
                        for dec in non_property_decorators)
                    )):
                    errors['PROPERTY IN METHODS SECTION'].append(line_num)
            elif 'PROPERTIES' in current_header:
                if (
                    lines[line_num].startswith('    def ') and
                    not lines[line_num - 1].startswith('    @')
                    ):
                    errors['METHOD IN PROPERTIES SECTION'].append(line_num)
        return errors



# -*- coding: utf-8 -*-


def select(argument=None):
    r'''Selects `argument` or makes empty selector.

    ..  container:: example

        Selects first two notes in staff:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> selection = select(staff[:2])
            >>> for note in selection:
            ...     override(note).note_head.color = 'red'

        ::

            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \once \override NoteHead.color = #red
                c'4
                \once \override NoteHead.color = #red
                d'4
                e'4
                f'4
            }

    ..  container:: example

        Initializes empty selector:

        ::

            >>> select()
            Selector()

    Returns selection when `argument` is not none.

    Returns selector when `argument` is none.
    '''
    from abjad.tools import scoretools
    from abjad.tools import selectiontools
    from abjad.tools import selectortools
    from abjad.tools import spannertools
    if argument is None:
        return selectortools.Selector()
    elif isinstance(argument, scoretools.Component):
        return selectiontools.Selection(argument)
    elif hasattr(argument, '_music'):
        music = argument._music
        return selectiontools.Selection(music)
    elif isinstance(argument, spannertools.Spanner):
        music = argument._components
        return selectiontools.Selection(music)
    return selectiontools.Selection(argument)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.pitchtools import *


def test_pitchtools_RegistrationComponent___repr___01():
    r'''Reprs are evaluable.
    '''

    component_1 = pitchtools.RegistrationComponent('[A0, C8]', 15)
    component_2 = eval(repr(component_1))

    assert component_1 == component_2



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Selection__give_dominant_spanners_01():
    r'''Find spanners that dominate donor_components.
    Apply dominant spanners to recipient_components.
    Withdraw donor_components from spanners.
    The operation can mangle spanners.
    Remove donor_components from parentage immediately after.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    crescendo = Crescendo()
    attach(crescendo, voice[:])
    beam = Beam()
    attach(beam, voice[:2])
    slur = Slur()
    attach(slur, voice[1:3])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \<
            d'8 ] (
            e'8 )
            f'8 \!
        }
        '''
        )

    recipient = Voice("c'16 c'16 c'16")
    beam = Beam()
    attach(beam, recipient[:])

    assert format(recipient) == stringtools.normalize(
        r'''
        \new Voice {
            c'16 [
            c'16
            c'16 ]
        }
        '''
        )

    voice[1:3]._give_dominant_spanners(recipient[:])


    "Both crescendo and beam are now discontiguous."

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \<
            d'8 ]
            e'8
            f'8 \!
        }
        '''
        )

    assert not inspect_(voice).is_well_formed()

    "Slur is contiguous but recipient participates in discont. cresc."

    assert format(recipient) == stringtools.normalize(
        r'''
        \new Voice {
            c'16 [ (
            c'16
            c'16 ] )
        }
        '''
        )

    assert not inspect_(recipient).is_well_formed()


# -*- coding: utf-8 -*-
import inspect
from abjad.tools import expressiontools
from abjad.tools.datastructuretools.TypedList import TypedList
from abjad.tools.markuptools.Markup import Markup


class MarkupList(TypedList):
    '''Markup list.

    ..  container:: example

        Initializes from strings:

        ..  container:: example

            ::

                >>> markups = ['Allegro', 'assai']
                >>> markup_list = MarkupList(markups)
                >>> f(markup_list)
                markuptools.MarkupList(
                    items=[
                        markuptools.Markup(
                            contents=['Allegro'],
                            ),
                        markuptools.Markup(
                            contents=['assai'],
                            ),
                        ],
                    )

            ::

                >>> show(markup_list) # doctest: +SKIP

            ..  doctest::

                >>> f(markup_list.__illustrate__().items[-1])
                \markup {
                    \column
                        {
                            Allegro
                            assai
                        }
                    }

        ..  container:: example expression

            ::

                >>> expression = Expression().markup_list()
                >>> markup_list = expression(['Allegro', 'assai'])
                >>> f(markup_list)
                markuptools.MarkupList(
                    items=[
                        markuptools.Markup(
                            contents=['Allegro'],
                            ),
                        markuptools.Markup(
                            contents=['assai'],
                            ),
                        ],
                    )

            ::

                >>> show(markup_list) # doctest: +SKIP

            ..  doctest::

                >>> f(markup_list.__illustrate__().items[-1])
                \markup {
                    \column
                        {
                            Allegro
                            assai
                        }
                    }

    Markup list implement the list interface and are mutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_expression',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        items=None,
        item_class=None,
        keep_sorted=None,
        ):
        from abjad.tools import markuptools
        self._expression = None
        item_class = item_class or markuptools.Markup
        TypedList.__init__(
            self,
            item_class=item_class,
            items=items,
            keep_sorted=keep_sorted,
            )

    ### SPECIAL METHODS ###

    def __contains__(self, item):
        r'''Is true when markup markup list contains `item`.
        Otherwise false.

        ..  container:: example

            ..  container:: example

                ::

                    >>> markups = ['Allegro', 'assai']
                    >>> markup_list = MarkupList(markups)

                ::

                    >>> 'assai' in markup_list
                    True

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> markup_list = expression(['Allegro', 'assai'])

                ::

                    >>> 'assai' in markup_list
                    True

        Returns true or false.
        '''
        superclass = super(MarkupList, self)
        return superclass.__contains__(item)

    def __format__(self, format_specification=''):
        r'''Formats markup list.

        ..  container:: example

            ..  container:: example

                Formats markup list:

                ::


                    >>> markups = ['Allegro', 'assai']
                    >>> markup_list = MarkupList(markups)
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['assai'],
                                ),
                            ],
                        )

            ..  container:: example expression

                Formats markup list expression:

                ::

                    >>> expression = Expression().markup_list()
                    >>> f(expression)
                    expressiontools.Expression(
                        callbacks=[
                            expressiontools.Expression(
                                evaluation_template='abjad.markuptools.MarkupList',
                                is_initializer=True,
                                ),
                            ],
                        proxy_class=markuptools.MarkupList,
                        )

        Returns string.
        '''
        superclass = super(MarkupList, self)
        return superclass.__format__(format_specification=format_specification)

    def __iadd__(self, argument):
        r'''Changes items in `argument` to items and extends markup list.

        ..  container:: example

            ..  container:: example

                ::

                    >>> markup_list = MarkupList()
                    >>> markup_list.extend(['Allegro', 'assai'])
                    >>> markup_list += ['ma', 'non', 'troppo']
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['assai'],
                                ),
                            markuptools.Markup(
                                contents=['ma'],
                                ),
                            markuptools.Markup(
                                contents=['non'],
                                ),
                            markuptools.Markup(
                                contents=['troppo'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

                ..  doctest::

                    >>> f(markup_list.__illustrate__().items[-1])
                    \markup {
                        \column
                            {
                                Allegro
                                assai
                                ma
                                non
                                troppo
                            }
                        }

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression += ['ma', 'non', 'troppo']
                    >>> markup_list = expression(['Allegro', 'assai'])
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['assai'],
                                ),
                            markuptools.Markup(
                                contents=['ma'],
                                ),
                            markuptools.Markup(
                                contents=['non'],
                                ),
                            markuptools.Markup(
                                contents=['troppo'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

        Returns none.
        '''
        if self._expression:
            return self._update_expression(
                inspect.currentframe(),
                force_return=True,
                )
        superclass = super(MarkupList, self)
        return superclass.__iadd__(argument)

    def __illustrate__(self):
        r'''Illustrates markup markup list.

        ..  container:: example

            ..  container:: example

                ::

                    >>> markups = ['Allegro', 'assai']
                    >>> markup_list = MarkupList(markups)
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['assai'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

                ..  doctest::

                    >>> f(markup_list.__illustrate__().items[-1])
                    \markup {
                        \column
                            {
                                Allegro
                                assai
                            }
                        }

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> markup_list = expression(['Allegro', 'assai'])
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['assai'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

                ..  doctest::

                    >>> f(markup_list.__illustrate__().items[-1])
                    \markup {
                        \column
                            {
                                Allegro
                                assai
                            }
                        }

        Returns LilyPond file.
        '''
        import abjad
        lilypond_file = abjad.LilyPondFile.new()
        for name in ('layout', 'paper', 'score'):
            block = lilypond_file[name]
            lilypond_file.items.remove(block)
        lilypond_file.header_block.tagline = False
        markup = abjad.Markup.column(list(self))
        lilypond_file.items.append(markup)
        return lilypond_file

    def __setitem__(self, i, argument):
        r'''Sets item `i` equal to `argument`.

        ..  container:: example

            ..  container:: example

                ::

                    >>> markup_list = MarkupList()
                    >>> markup_list.extend(['Allegro', 'assai'])
                    >>> markup_list[-1] = 'non troppo'
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['non troppo'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

                ..  doctest::

                    >>> f(markup_list.__illustrate__().items[-1])
                    \markup {
                        \column
                            {
                                Allegro
                                "non troppo"
                            }
                        }

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.__setitem__(-1, 'non troppo')
                    >>> markup_list = expression(['Allegro', 'assai'])
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['non troppo'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

                ..  doctest::

                    >>> f(markup_list.__illustrate__().items[-1])
                    \markup {
                        \column
                            {
                                Allegro
                                "non troppo"
                            }
                        }

        Returns none.
        '''
        if self._expression:
            return self._update_expression(
                inspect.currentframe(),
                force_return=True,
                )
        superclass = super(MarkupList, self)
        return superclass.__setitem__(i, argument)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        import abjad
        agent = abjad.systemtools.StorageFormatAgent(self)
        names = list(agent.signature_keyword_names)
        if self.item_class is abjad.Markup:
            names.remove('item_class')
        return abjad.systemtools.FormatSpecification(
            client=self,
            storage_format_kwargs_names=names,
            )

    def _update_expression(self, frame, force_return=None):
        #import abjad
        from abjad.tools import expressiontools
        callback = expressiontools.Expression._frame_to_callback(
            frame,
            force_return=force_return,
            )
        return self._expression.append_callback(callback)

    ### PUBLIC PROPERTIES ###

    @property
    def item_class(self):
        r'''Gets markup list item class.

        ..  container:: example

            ::

                >>> MarkupList().item_class
                <class 'abjad.tools.markuptools.Markup.Markup'>

        Returns markup class.
        '''
        superclass = super(MarkupList, self)
        return superclass.item_class

    @property
    def items(self):
        r'''Gets markup list items.

        ..  container:: example

            ..  container:: example

                Initializes items positionally:

                ::

                    >>> items = ['Allegro', 'assai']
                    >>> markup_list = MarkupList(items)
                    >>> for item in markup_list.items:
                    ...     item
                    ...
                    Markup(contents=['Allegro'])
                    Markup(contents=['assai'])

                Initializes items from keyword:

                ::

                    >>> items = ['Allegro', 'assai']
                    >>> markup_list = MarkupList(items=items)
                    >>> for item in markup_list.items:
                    ...     item
                    ...
                    Markup(contents=['Allegro'])
                    Markup(contents=['assai'])

            ..  container:: example expression

                Initializes items positionally:

                ::

                    >>> items = ['Allegro', 'assai']
                    >>> expression = Expression().markup_list()
                    >>> markup_list = expression(items)
                    >>> for item in markup_list.items:
                    ...     item
                    ...
                    Markup(contents=['Allegro'])
                    Markup(contents=['assai'])

                Initializes items from keyword:

                ::

                    >>> items = ['Allegro', 'assai']
                    >>> expression = Expression().markup_list()
                    >>> markup_list = expression(items=items)
                    >>> for item in markup_list.items:
                    ...     item
                    ...
                    Markup(contents=['Allegro'])
                    Markup(contents=['assai'])

        Returns tuple.
        '''
        superclass = super(MarkupList, self)
        return superclass.items

    @property
    def keep_sorted(self):
        r'''Is true when markup list keeps markups sorted.
        Otherwise false.

        ..  container:: example

            Keeps markup sorted:

            ::

                >>> markups = ['Allegro', 'assai']
                >>> markup_list = MarkupList(keep_sorted=True)
                >>> markup_list.append('assai')
                >>> markup_list.append('Allegro')
                >>> show(markup_list) # doctest: +SKIP

            ..  doctest::

                >>> f(markup_list.__illustrate__().items[-1])
                \markup {
                    \column
                        {
                            Allegro
                            assai
                        }
                    }

        ..  container:: example

            Does not keep markup sorted:

            ::

                >>> markups = ['Allegro', 'assai']
                >>> markup_list = MarkupList()
                >>> markup_list.append('assai')
                >>> markup_list.append('Allegro')
                >>> show(markup_list) # doctest: +SKIP

            ..  doctest::

                >>> f(markup_list.__illustrate__().items[-1])
                \markup {
                    \column
                        {
                            assai
                            Allegro
                        }
                    }

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        superclass = super(MarkupList, self)
        return superclass.keep_sorted

    ### PUBLIC METHODS ###

    def append(self, item):
        r'''Appends `item` to markup list.

        ..  container:: example

            ..  container:: example

                ::

                    >>> markup_list = MarkupList(['Allegro'])
                    >>> markup_list.append('assai')
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['assai'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.append('assai')
                    >>> markup_list = expression(['Allegro'])
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['assai'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

        Returns none.
        '''
        if self._expression:
            return self._update_expression(
                inspect.currentframe(),
                force_return=True,
                )
        superclass = super(MarkupList, self)
        superclass.append(item)

    def center_column(self, direction=None):
        r'''LilyPond ``\center-column`` markup command.

        ..  container:: example

            ..  container:: example

                ::

                    >>> city = Markup('Los Angeles')
                    >>> date = Markup('May - August 2014')
                    >>> markups = [city, date]
                    >>> markup_list = MarkupList(markups)
                    >>> markup = markup_list.center_column(direction=Up)
                    >>> f(markup)
                    ^ \markup {
                        \center-column
                            {
                                "Los Angeles"
                                "May - August 2014"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.center_column(direction=Up)
                    >>> city = Markup('Los Angeles')
                    >>> date = Markup('May - August 2014')
                    >>> markups = [city, date]
                    >>> markup = expression(markups)
                    >>> f(markup)
                    ^ \markup {
                        \center-column
                            {
                                "Los Angeles"
                                "May - August 2014"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

        Returns new markup.
        '''
        from abjad.tools import markuptools
        if self._expression:
            return self._update_expression(inspect.currentframe())
        contents = []
        for markup in self:
            string = markuptools.Markup._parse_markup_command_argument(markup)
            contents.append(string)
        command = markuptools.MarkupCommand('center-column', contents)
        return markuptools.Markup(contents=command, direction=direction)

    def column(self, direction=None):
        r'''LilyPond ``\column`` markup command.

        ..  container:: example

            ..  container:: example

                ::

                    >>> city = Markup('Los Angeles')
                    >>> date = Markup('May - August 2014')
                    >>> markup_list = MarkupList([city, date])
                    >>> markup = markup_list.column()
                    >>> f(markup)
                    \markup {
                        \column
                            {
                                "Los Angeles"
                                "May - August 2014"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.column()
                    >>> city = Markup('Los Angeles')
                    >>> date = Markup('May - August 2014')
                    >>> markup = expression([city, date])
                    >>> f(markup)
                    \markup {
                        \column
                            {
                                "Los Angeles"
                                "May - August 2014"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

        Returns new markup.
        '''
        import abjad
        if self._expression:
            return self._update_expression(inspect.currentframe())
        contents = []
        for markup in self:
            contents.extend(markup.contents)
        command = abjad.markuptools.MarkupCommand('column', contents)
        return abjad.Markup(contents=command, direction=direction)

    def combine(self, direction=None):
        r'''LilyPond ``\combine`` markup command.

        ..  container:: example

            ..  container:: example

                ::

                    >>> markup_one = Markup('Allegro assai')
                    >>> markup_two = Markup.draw_line(13, 0)
                    >>> markup_list = [markup_one, markup_two]
                    >>> markup_list = MarkupList(markup_list)
                    >>> markup = markup_list.combine(direction=Up)
                    >>> f(markup)
                    ^ \markup {
                        \combine
                            "Allegro assai"
                            \draw-line
                                #'(13 . 0)
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.combine(direction=Up)
                    >>> markup_1 = Markup('Allegro assai')
                    >>> markup_2 = Markup.draw_line(13, 0)
                    >>> markup_list = [markup_1, markup_2]
                    >>> markup = expression(markup_list)
                    >>> f(markup)
                    ^ \markup {
                        \combine
                            "Allegro assai"
                            \draw-line
                                #'(13 . 0)
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

        Returns new markup.
        '''
        import abjad
        if self._expression:
            return self._update_expression(inspect.currentframe())
        if not len(self) == 2:
            message = 'markup list must be length 2: {!r}.'
            message = message.format(markup_list)
            raise Exception(message)
        markup_1, markup_2 = self.items
        contents_1 = abjad.Markup._parse_markup_command_argument(markup_1)
        contents_2 = abjad.Markup._parse_markup_command_argument(markup_2)
        command = abjad.markuptools.MarkupCommand(
            'combine', contents_1, contents_2)
        return abjad.Markup(contents=command, direction=direction)

    def concat(self, direction=None):
        r'''LilyPond ``\concat`` markup command.

        ..  container:: example

            ..  container:: example

                ::

                    >>> downbow = Markup.musicglyph('scripts.downbow')
                    >>> hspace = Markup.hspace(1)
                    >>> upbow = Markup.musicglyph('scripts.upbow')
                    >>> markups = [downbow, hspace, upbow]
                    >>> markup_list = MarkupList(markups)
                    >>> markup = markup_list.concat(direction=Up)
                    >>> f(markup)
                    ^ \markup {
                        \concat
                            {
                                \musicglyph
                                    #"scripts.downbow"
                                \hspace
                                    #1
                                \musicglyph
                                    #"scripts.upbow"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.concat(direction=Up)
                    >>> downbow = Markup.musicglyph('scripts.downbow')
                    >>> hspace = Markup.hspace(1)
                    >>> upbow = Markup.musicglyph('scripts.upbow')
                    >>> markups = [downbow, hspace, upbow]
                    >>> markup = expression(markups)
                    >>> f(markup)
                    ^ \markup {
                        \concat
                            {
                                \musicglyph
                                    #"scripts.downbow"
                                \hspace
                                    #1
                                \musicglyph
                                    #"scripts.upbow"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

        Returns new markup.
        '''
        from abjad.tools import markuptools
        if self._expression:
            return self._update_expression(inspect.currentframe())
        result = []
        for markup in self:
            contents = markuptools.Markup._parse_markup_command_argument(
                markup)
            result.append(contents)
        command = markuptools.MarkupCommand('concat', result)
        return markuptools.Markup(contents=command, direction=direction)

    def count(self, item):
        r'''Counts `item` in markup list.

        ..  container:: example

            ::

                >>> markup_list = MarkupList()
                >>> markup_list.extend(['Allegro', 'assai'])

            ::

                >>> show(markup_list) # doctest: +SKIP

            ::

                >>> markup_list.count('Allegro')
                1
                >>> markup_list.count('assai')
                1
                >>> markup_list.count('ma non troppo')
                0

        Returns none.
        '''
        superclass = super(MarkupList, self)
        return superclass.count(item)

    def extend(self, items):
        r'''Extends markup list with `items`.

        ..  container:: example

            ..  container:: example

                ::

                    >>> markup_list = MarkupList()
                    >>> markup_list.extend(['Allegro', 'assai'])
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['assai'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

                ..  doctest::
            
                    >>> f(markup_list.__illustrate__().items[-1])
                    \markup {
                        \column
                            {
                                Allegro
                                assai
                            }
                        }

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.extend(['assai'])
                    >>> markup_list = expression(['Allegro'])
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['assai'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

                ..  doctest::

                    >>> f(markup_list.__illustrate__().items[-1])
                    \markup {
                        \column
                            {
                                Allegro
                                assai
                            }
                        }

        Returns none.
        '''
        if self._expression:
            return self._update_expression(
                inspect.currentframe(),
                force_return=True,
                )
        superclass = super(MarkupList, self)
        superclass.extend(items)

    def index(self, item):
        r'''Gets index of `item` in markup list.

        ..  container:: example

            ::

                >>> markup_list = MarkupList()
                >>> markup_list.extend(['Allegro', 'assai'])
                >>> f(markup_list)
                markuptools.MarkupList(
                    items=[
                        markuptools.Markup(
                            contents=['Allegro'],
                            ),
                        markuptools.Markup(
                            contents=['assai'],
                            ),
                        ],
                    )

            ::

                >>> show(markup_list) # doctest: +SKIP

            ..  doctest::

                >>> f(markup_list.__illustrate__().items[-1])
                \markup {
                    \column
                        {
                            Allegro
                            assai
                        }
                    }

            ::

                >>> markup_list.index('Allegro')
                0
                >>> markup_list.index('assai')
                1

        Returns none.
        '''
        superclass = super(MarkupList, self)
        return superclass.index(item)

    def insert(self, i, item):
        r'''Inserts `item` in markup markup list.

        ..  container:: example

            ..  container:: example

                ::

                    >>> markup_list = MarkupList(['assai'])
                    >>> markup_list.insert(0, 'Allegro')
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['assai'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.insert(0, 'Allegro')
                    >>> markup_list = expression(['assai'])
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            markuptools.Markup(
                                contents=['assai'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

        Returns markup class.
        '''
        if self._expression:
            return self._update_expression(
                inspect.currentframe(),
                force_return=True,
                )
        superclass = super(MarkupList, self)
        superclass.insert(i, item)

    def left_column(self, direction=None):
        r'''LilyPond ``\left-column`` markup command.

        ..  container:: example

            ..  container:: example

                ::

                    >>> city = Markup('Los Angeles')
                    >>> date = Markup('May - August 2014')
                    >>> markup_list = MarkupList([city, date])
                    >>> markup = markup_list.left_column()
                    >>> f(markup)
                    \markup {
                        \left-column
                            {
                                "Los Angeles"
                                "May - August 2014"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.left_column()
                    >>> city = Markup('Los Angeles')
                    >>> date = Markup('May - August 2014')
                    >>> markup = expression([city, date])
                    >>> f(markup)
                    \markup {
                        \left-column
                            {
                                "Los Angeles"
                                "May - August 2014"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

        Returns new markup.
        '''
        import abjad
        if self._expression:
            return self._update_expression(inspect.currentframe())
        contents = []
        for markup in self:
            contents.append(abjad.Markup._parse_markup_command_argument(markup))
        command = abjad.markuptools.MarkupCommand('left-column', contents)
        return abjad.Markup(contents=command, direction=direction)

    def line(self, direction=None):
        r'''LilyPond ``\line`` markup command.

        ..  container:: example

            ..  container:: example

                ::

                    >>> markups = ['Allegro', 'assai']
                    >>> markup_list = MarkupList(markups)
                    >>> markup = markup_list.line()
                    >>> f(markup)
                    \markup {
                        \line
                            {
                                Allegro
                                assai
                            }
                        }


                ::

                    >>> show(markup) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.line()
                    >>> markup = expression(['Allegro', 'assai'])
                    >>> f(markup)
                    \markup {
                        \line
                            {
                                Allegro
                                assai
                            }
                        }


                ::

                    >>> show(markup) # doctest: +SKIP

        Returns new markup.
        '''
        import abjad
        if self._expression:
            return self._update_expression(inspect.currentframe())
        contents = []
        for markup in self:
            contents.extend(markup.contents)
        command = abjad.markuptools.MarkupCommand('line', contents)
        return abjad.Markup(contents=command, direction=direction)

    def overlay(self, direction=None):
        r'''LilyPond ``\overlay`` markup command.

        ..  container:: example

            ..  container:: example

                ::

                    >>> city = Markup('Los Angeles')
                    >>> date = Markup('May - August 2014')
                    >>> markup_list = MarkupList([city, date])
                    >>> markup = markup_list.overlay(direction=Up)
                    >>> f(markup)
                    ^ \markup {
                        \overlay
                            {
                                "Los Angeles"
                                "May - August 2014"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.overlay(direction=Up)
                    >>> city = Markup('Los Angeles')
                    >>> date = Markup('May - August 2014')
                    >>> markup = expression([city, date])
                    >>> f(markup)
                    ^ \markup {
                        \overlay
                            {
                                "Los Angeles"
                                "May - August 2014"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

        Returns new markup.
        '''
        import abjad
        if self._expression:
            return self._update_expression(inspect.currentframe())
        contents = []
        for markup in self:
            contents.append(abjad.Markup._parse_markup_command_argument(markup))
        command = abjad.markuptools.MarkupCommand('overlay', contents)
        return abjad.Markup(contents=command, direction=direction)

    def pop(self, i=-1):
        r'''Pops item `i` from markup list.

        ..  container:: example

            ..  container:: example

                ::

                    >>> markup_list = MarkupList()
                    >>> markup_list.extend(['Allegro', 'assai'])
                    >>> markup_list.pop()
                    Markup(contents=['assai'])

                ::

                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.pop()
                    >>> markup_list = expression(['Allegro', 'assai'])

                ::

                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

        Returns none.
        '''
        if self._expression:
            return self._update_expression(
                inspect.currentframe(),
                force_return=True,
                )
        superclass = super(MarkupList, self)
        return superclass.pop(i=i)

    def remove(self, item):
        r'''Removes `item` from markup list.

        ..  container:: example

            ..  container:: example

                ::

                    >>> markup_list = MarkupList()
                    >>> markup_list.extend(['Allegro', 'assai'])
                    >>> markup_list.remove('assai')
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

                ..  doctest::

                    >>> f(markup_list.__illustrate__().items[-1])
                    \markup {
                        \column
                            {
                                Allegro
                            }
                        }

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.remove('assai')
                    >>> markup_list = expression(['Allegro', 'assai'])
                    >>> f(markup_list)
                    markuptools.MarkupList(
                        items=[
                            markuptools.Markup(
                                contents=['Allegro'],
                                ),
                            ],
                        )

                ::

                    >>> show(markup_list) # doctest: +SKIP

                ..  doctest::

                    >>> f(markup_list.__illustrate__().items[-1])
                    \markup {
                        \column
                            {
                                Allegro
                            }
                        }

        Returns none.
        '''
        if self._expression:
            return self._update_expression(
                inspect.currentframe(),
                force_return=True,
                )
        superclass = super(MarkupList, self)
        superclass.remove(item)

    def right_column(self, direction=None):
        r'''LilyPond ``\right-column`` markup command.

        ..  container:: example

            ..  container:: example

                ::

                    >>> city = Markup('Los Angeles')
                    >>> date = Markup('May - August 2014')
                    >>> markup_list = MarkupList([city, date])
                    >>> markup = markup_list.right_column()
                    >>> f(markup)
                    \markup {
                        \right-column
                            {
                                "Los Angeles"
                                "May - August 2014"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

            ..  container:: example expression

                ::

                    >>> expression = Expression().markup_list()
                    >>> expression = expression.right_column()
                    >>> city = Markup('Los Angeles')
                    >>> date = Markup('May - August 2014')
                    >>> markup = expression([city, date])
                    >>> f(markup)
                    \markup {
                        \right-column
                            {
                                "Los Angeles"
                                "May - August 2014"
                            }
                        }

                ::

                    >>> show(markup) # doctest: +SKIP

        Returns new markup.
        '''
        import abjad
        if self._expression:
            return self._update_expression(inspect.currentframe())
        contents = []
        for markup in self:
            contents.append(abjad.Markup._parse_markup_command_argument(markup))
        command = abjad.markuptools.MarkupCommand('right-column', contents)
        return abjad.Markup(contents=command, direction=direction)



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_KeySignature___eq___01():

    key_signature1 = KeySignature('g', 'major')
    key_signature2 = KeySignature('g', 'major')
    key_signature3 = KeySignature('g', 'minor')

    assert key_signature1 == key_signature2
    assert not key_signature2 == key_signature3
    assert not key_signature3 == key_signature1



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools.inspect_ import inspect_


class Glissando(Spanner):
    r'''Glissando.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> glissando = spannertools.Glissando()
            >>> attach(glissando, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'8 \glissando
                d'8 \glissando
                e'8 \glissando
                f'8
            }

    ..  container:: example

        Glissando avoids BendAfter indicators.

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> bend_after = indicatortools.BendAfter()
            >>> attach(bend_after, staff[1], is_annotation=True)
            >>> glissando = spannertools.Glissando()
            >>> attach(glissando, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'8 \glissando
                d'8 - \bendAfter #'-4.0
                e'8 \glissando
                f'8
            }

    Formats notes and chords with LilyPond ``\glissando`` command.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_allow_repeat_pitches',
        '_allow_ties',
        '_parenthesize_repeated_pitches',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        allow_repeat_pitches=False,
        allow_ties=False,
        overrides=None,
        parenthesize_repeated_pitches=False,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        allow_ties = bool(allow_ties)
        allow_repeat_pitches = bool(allow_repeat_pitches)
        parenthesize_repeated_pitches = bool(parenthesize_repeated_pitches)
        self._allow_ties = allow_ties
        self._allow_repeat_pitches = allow_repeat_pitches
        self._parenthesize_repeated_pitches = parenthesize_repeated_pitches

    ### PRIVATE METHODS ###

    def _attachment_test_all(self, argument):
        return self._at_least_two_leaves(argument)

    def _copy_keyword_args(self, new):
        Spanner._copy_keyword_args(self, new)
        new._allow_repeat_pitches = self.allow_repeat_pitches
        new._allow_ties = self.allow_ties
        new._parenthesize_repeated_pitches = self.parenthesize_repeated_pitches

    def _get_annotations(self, leaf):
        inspector = inspect_(leaf)
        bend_after = None
        prototype = indicatortools.BendAfter
        if inspector.has_indicator(prototype):
            bend_after = inspector.get_indicator(prototype)
        return bend_after

    def _get_lilypond_format_bundle(self, leaf):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        prototype = (scoretools.Chord, scoretools.Note)
        bend_after = self._get_annotations(leaf)
        should_attach_glissando = False
        if not self._is_my_first_leaf(leaf):
            if self.parenthesize_repeated_pitches:
                if not self._previous_leaf_changes_current_pitch(leaf):
                    self._parenthesize_leaf(leaf)
        if bend_after:
            lilypond_format_bundle.update(
                bend_after._get_lilypond_format_bundle(),
                )
        elif self._is_my_last_leaf(leaf):
            pass
        elif not isinstance(leaf, prototype):
            pass
        elif self.allow_repeat_pitches and self.allow_ties:
            should_attach_glissando = True
        elif self.allow_repeat_pitches and not self.allow_ties:
            should_attach_glissando = self._is_last_in_tie_chain(leaf)
        elif not self.allow_repeat_pitches and self.allow_ties:
            if self._next_leaf_changes_current_pitch(leaf):
                should_attach_glissando = True
        elif (
            not self.allow_repeat_pitches and
            not self.allow_ties):
            if self._next_leaf_changes_current_pitch(leaf):
                if self._is_last_in_tie_chain(leaf):
                    should_attach_glissando = True
        if should_attach_glissando:
            lilypond_format_bundle.right.spanner_starts.append('\glissando')
        return lilypond_format_bundle

    @staticmethod
    def _is_last_in_tie_chain(leaf):
        logical_tie = inspect_(leaf).get_logical_tie()
        return leaf is logical_tie[-1]

    @staticmethod
    def _next_leaf_changes_current_pitch(leaf):
        next_leaf = inspect_(leaf).get_leaf(n=1)
        if (isinstance(leaf, scoretools.Note) and
            isinstance(next_leaf, scoretools.Note) and
            leaf.written_pitch == next_leaf.written_pitch):
            return False
        elif (isinstance(leaf, scoretools.Chord) and
            isinstance(next_leaf, scoretools.Chord) and
            leaf.written_pitches == next_leaf.written_pitches):
            return False
        return True

    @staticmethod
    def _parenthesize_leaf(leaf):
        if isinstance(leaf, scoretools.Note):
            leaf.note_head.is_parenthesized = True
        elif isinstance(leaf, scoretools.Chord):
            for note_head in leaf.note_heads:
                note_head.is_parenthesized = True

    @staticmethod
    def _previous_leaf_changes_current_pitch(leaf):
        previous_leaf = inspect_(leaf).get_leaf(n=-1)
        if (isinstance(leaf, scoretools.Note) and
            isinstance(previous_leaf, scoretools.Note) and
            leaf.written_pitch == previous_leaf.written_pitch):
            return False
        elif (isinstance(leaf, scoretools.Chord) and
            isinstance(previous_leaf, scoretools.Chord) and
            leaf.written_pitches == previous_leaf.written_pitches):
            return False
        return True

    ### PUBLIC PROPERTIES ###

    @property
    def allow_repeat_pitches(self):
        r'''Is true when glissando should allow repeated pitches.
        Otherwise false.

        ..  container:: example

            Does not allow repeated pitches:

            ::

                >>> staff = Staff("a8 a8 b8 ~ b8 c'8 c'8 d'8 ~ d'8")
                >>> glissando = Glissando(
                ...     allow_repeat_pitches=False,
                ...     )
                >>> attach(glissando, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    a8
                    a8 \glissando
                    b8 ~
                    b8 \glissando
                    c'8
                    c'8 \glissando
                    d'8 ~
                    d'8
                }

            This is default behavior.

        ..  container:: example

            Allows repeated pitches (but not ties):

            ::

                >>> staff = Staff("a8 a8 b8 ~ b8 c'8 c'8 d'8 ~ d'8")
                >>> glissando = Glissando(
                ...     allow_repeat_pitches=True,
                ...     )
                >>> attach(glissando, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    a8 \glissando
                    a8 \glissando
                    b8 ~
                    b8 \glissando
                    c'8 \glissando
                    c'8 \glissando
                    d'8 ~
                    d'8
                }

        ..  container:: example

            Allows both repeated pitches and ties:

            ::

                >>> staff = Staff("a8 a8 b8 ~ b8 c'8 c'8 d'8 ~ d'8")
                >>> glissando = Glissando(
                ...     allow_repeat_pitches=True,
                ...     allow_ties=True,
                ...     )
                >>> attach(glissando, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    a8 \glissando
                    a8 \glissando
                    b8 ~ \glissando
                    b8 \glissando
                    c'8 \glissando
                    c'8 \glissando
                    d'8 ~ \glissando
                    d'8
                }

        Ties are excluded when repeated pitches are not allowed because all
        ties comprise repeated pitches.

        Defaults to false.
        '''
        return self._allow_repeat_pitches

    @property
    def allow_ties(self):
        r'''Is true when glissando should allow ties. Otherwise false.

        ..  container:: example

            Does not allow repeated pitches (including ties):

            ::

                >>> staff = Staff("a8 a8 b8 ~ b8 c'8 c'8 d'8 ~ d'8")
                >>> glissando = Glissando(
                ...     allow_repeat_pitches=False,
                ...     )
                >>> attach(glissando, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    a8
                    a8 \glissando
                    b8 ~
                    b8 \glissando
                    c'8
                    c'8 \glissando
                    d'8 ~
                    d'8
                }

            This is default behavior.

        ..  container:: example

            Allows repeated pitches (but not ties):

            ::

                >>> staff = Staff("a8 a8 b8 ~ b8 c'8 c'8 d'8 ~ d'8")
                >>> glissando = Glissando(
                ...     allow_repeat_pitches=True,
                ...     )
                >>> attach(glissando, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    a8 \glissando
                    a8 \glissando
                    b8 ~
                    b8 \glissando
                    c'8 \glissando
                    c'8 \glissando
                    d'8 ~
                    d'8
                }

        ..  container:: example

            Allows both repeated pitches and ties:

            ::

                >>> staff = Staff("a8 a8 b8 ~ b8 c'8 c'8 d'8 ~ d'8")
                >>> glissando = Glissando(
                ...     allow_repeat_pitches=True,
                ...     allow_ties=True,
                ...     )
                >>> attach(glissando, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    a8 \glissando
                    a8 \glissando
                    b8 ~ \glissando
                    b8 \glissando
                    c'8 \glissando
                    c'8 \glissando
                    d'8 ~ \glissando
                    d'8
                }

        Ties are excluded when repeated pitches are not allowed because all
        ties comprise repeated pitches.

        Defaults to false.
        '''
        return self._allow_ties

    @property
    def parenthesize_repeated_pitches(self):
        r'''Is true when glissando should parenthesize repeated pitches.
        Otherwise false.

        ..  container:: example

            Does not parenthesize repeated pitches:

            ::

                >>> staff = Staff("a8 a8 b8 ~ b8 c'8 c'8 d'8 ~ d'8")
                >>> glissando = Glissando()
                >>> attach(glissando, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    a8
                    a8 \glissando
                    b8 ~
                    b8 \glissando
                    c'8
                    c'8 \glissando
                    d'8 ~
                    d'8
                }

            This is default behavior.

        ..  container:: example

            Spans and parenthesizes repeated pitches:

            ::

                >>> staff = Staff("a8 a8 b8 ~ b8 c'8 c'8 d'8 ~ d'8")
                >>> glissando = Glissando(
                ...     allow_repeat_pitches=True,
                ...     parenthesize_repeated_pitches=True,
                ...     )
                >>> attach(glissando, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    a8 \glissando
                    \parenthesize
                    a8 \glissando
                    b8 ~
                    \parenthesize
                    b8 \glissando
                    c'8 \glissando
                    \parenthesize
                    c'8 \glissando
                    d'8 ~
                    \parenthesize
                    d'8
                }

        ..  container:: example

            Parenthesizes (but does not span) repeated pitches:

            ::

                >>> staff = Staff("a8 a8 b8 ~ b8 c'8 c'8 d'8 ~ d'8")
                >>> glissando = Glissando(
                ...     parenthesize_repeated_pitches=True,
                ...     )
                >>> attach(glissando, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    a8
                    \parenthesize
                    a8 \glissando
                    b8 ~
                    \parenthesize
                    b8 \glissando
                    c'8
                    \parenthesize
                    c'8 \glissando
                    d'8 ~
                    \parenthesize
                    d'8
                }

        Defaults to false.
        '''
        return self._parenthesize_repeated_pitches



# -*- coding: utf-8 -*-
from __future__ import print_function
import importlib
import inspect
import types
from abjad.tools.abctools.AbjadObject import AbjadObject


class InheritanceGraph(AbjadObject):
    r'''Generates a graph of a class or collection of
    classes as a dictionary of parent-children relationships:

    ::

        >>> class A(object): pass
        ...
        >>> class B(A): pass
        ...
        >>> class C(B): pass
        ...
        >>> class D(B): pass
        ...
        >>> class E(C, D): pass
        ...
        >>> class F(A): pass
        ...

    ::

        >>> graph = documentationtools.InheritanceGraph(addresses=(F, E))

    ``InheritanceGraph`` may be instantiated from one or more instances,
    classes or modules. If instantiated from a module, all public classes
    in that module will be taken into the graph.

    A `root_class` keyword may be defined at instantiation, which filters out
    all classes from the graph which do not inherit from that `root_class`
    (or are not already the `root_class`):

    ::

        >>> graph = documentationtools.InheritanceGraph(
        ...     (A, B, C, D, E, F),
        ...     root_addresses=(B,),
        ...     )

    The class is intended for use in documenting packages.

    To document all of Abjad, use this formulation:

    ::

        >>> graph = documentationtools.InheritanceGraph(
        ...     addresses=('abjad',),
        ...     )

    To document only those classes descending from Container,
    use this formulation:

    ::

        >>> graph = documentationtools.InheritanceGraph(
        ...     addresses=('abjad',),
        ...     root_addresses=(Container,),
        ...     )

    To document only those classes whose lineage pass through scoretools,
    use this formulation:

    ::

        >>> graph = documentationtools.InheritanceGraph(
        ...     addresses=('abjad',),
        ...     lineage_addresses=(scoretools,),
        ...     )

    When creating the Graphviz representation, classes in the inheritance
    graph may be hidden, based on their distance from any defined lineage
    class:

    ::

        >>> graph = documentationtools.InheritanceGraph(
        ...     addresses=('abjad',),
        ...     lineage_addresses=(instrumenttools.Instrument,),
        ...     lineage_prune_distance=1,
        ...     )

    Returns ``InheritanceGraph`` instance.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Documenters'

    __slots__ = (
        '_addresses',
        '_child_parents_mapping',
        '_immediate_classes',
        '_lineage_addresses',
        '_lineage_classes',
        '_lineage_distance_mapping',
        '_lineage_prune_distance',
        '_parent_children_mapping',
        '_recurse_into_submodules',
        '_root_addresses',
        '_root_classes',
        '_use_clusters',
        '_use_groups',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        addresses=('abjad',),
        lineage_addresses=None,
        lineage_prune_distance=None,
        recurse_into_submodules=True,
        root_addresses=None,
        use_clusters=True,
        use_groups=True,
        ):
        r'''Find a lot of classes by examining the contents of module objects
        recursively and collect those classes into a big list:
            _collect_classes()

        Optionally, collect “lineage” and “root” classes, which affect how the
        graph will later be pruned. Construct two dictionaries:
            _build_basic_mappings()

        The keys in one are parent classes and the values are sets of immediate
        subclasses (inheriting directly from).

        And the keys in the other are classes and the values are sets of their
        immediate parent classes.

        The two dictionaries give you a graph with bidirectional relationships.

        Populate the two dictionaries by looking at the MROs of all previously
        collected classes.

        If any “lineage” classes were defined on init, prune the graph of all
        classes whose inheritance chains do not pass through those classes.

        Finally, on __graph__(), use the pruned mappings to build a
        GraphvizGraph.
        '''
        self._recurse_into_submodules = bool(recurse_into_submodules)
        if lineage_prune_distance is not None:
            lineage_prune_distance = int(lineage_prune_distance)
            assert 0 < lineage_prune_distance
        self._lineage_prune_distance = lineage_prune_distance
        self._use_clusters = bool(use_clusters)
        self._use_groups = bool(use_groups)

        # main addresses
        if addresses is None:
            addresses = ('abjad',)
        all_main_classes, main_immediate_classes, main_cached_addresses = \
            self._collect_classes(addresses, self.recurse_into_submodules)
        self._addresses = main_cached_addresses

        # lineage addresses
        if lineage_addresses is not None:
            all_lineage_classes, lineage_immediate_classes, lineage_cached_addresses = \
                self._collect_classes(lineage_addresses, False)
            self._lineage_addresses = lineage_cached_addresses
            self._lineage_classes = frozenset(all_lineage_classes)
        else:
            self._lineage_addresses = None
            self._lineage_classes = frozenset([])

        # root addresses
        if root_addresses is not None:
            all_root_classes, root_immediate_classes, root_cached_addresses = \
                self._collect_classes(root_addresses, False)
            self._root_addresses = root_cached_addresses
            self._root_classes = frozenset(all_root_classes)
        else:
            self._root_addresses = None
            self._root_classes = frozenset([object])

        child_parents_mapping, parent_children_mapping = \
            self._build_basic_mappings(all_main_classes)
        self._strip_nonlineage_classes(
            child_parents_mapping, parent_children_mapping)

        self._child_parents_mapping = child_parents_mapping
        self._parent_children_mapping = parent_children_mapping

        self._immediate_classes = main_immediate_classes.intersection(
            self._parent_children_mapping.keys())

        self._lineage_distance_mapping = self._find_lineage_distances()

    ### SPECIAL METHODS ###

    def __graph__(self, **keywords):
        r'''Graphviz graph of inheritance graph.
        '''
        from abjad.tools import graphtools

        class_nodes = {}

        graph = graphtools.GraphvizGraph(
            name='InheritanceGraph',
            attributes={
                'bgcolor': 'transparent',
                'color': 'lightslategrey',
                'fontname': 'Arial',
                'outputorder': 'edgesfirst',
                'overlap': 'prism',
                'penwidth': 2,
                #'ranksep': 0.5,
                'splines': 'spline',
                'style': ('dotted', 'rounded'),
                'truecolor': True,
                },
            edge_attributes={
                'color': 'lightsteelblue2',
                'penwidth': 2,
                },
            node_attributes={
                'colorscheme': 'pastel19',
                'fontname': 'Arial',
                'fontsize': 12,
                'penwidth': 2,
                'style': ('filled', 'rounded'),
                },
            )

        for current_class in sorted(self.parent_children_mapping,
            key=lambda x: (x.__module__, x.__name__)):
            pieces = self._get_class_name_pieces(current_class)

            try:
                cluster = graph[pieces[0]]
            except KeyError:
                cluster = graphtools.GraphvizSubgraph(
                    name=pieces[0],
                    attributes={
                        'label': pieces[0],
                    },
                    )
                graph.append(cluster)

            node_name = '{}.{}'.format(
                current_class.__module__,
                current_class.__name__,
                )
            node = graphtools.GraphvizNode(
                name=node_name,
                #name='.'.join(pieces),
                )
            node.attributes['label'] = pieces[-1]

            if current_class in self.immediate_classes:
                pass
            if current_class in self.root_classes:
                pass
            if inspect.isabstract(current_class):
                node.attributes['shape'] = 'oval'
                node.attributes['style'] = 'bold'
            else:
                node.attributes['shape'] = 'box'
            if current_class in self.lineage_classes:
                node.attributes['color'] = 'black'
                node.attributes['fontcolor'] = 'white'
                node.attributes['style'] = ('filled', 'rounded')

            if self.lineage_prune_distance is None:
                cluster.append(node)
                class_nodes[current_class] = node
            elif current_class not in self.lineage_distance_mapping:
                cluster.append(node)
                class_nodes[current_class] = node
            else:
                ok_distance = self.lineage_prune_distance + 1
                distance = self.lineage_distance_mapping[current_class]
                if distance < ok_distance:
                    cluster.append(node)
                    class_nodes[current_class] = node
                elif distance == ok_distance:
                    node.attributes['shape'] = 'invis'
                    node.attributes['style'] = 'transparent'
                    node.attributes['label'] = ' '
                    cluster.append(node)
                    class_nodes[current_class] = node
                elif ok_distance < distance:
                    pass

        distances = self.lineage_distance_mapping
        for parent in sorted(self.parent_children_mapping,
            key=lambda x: (x.__module__, x.__name__)):
            children = self.parent_children_mapping[parent]
            children = sorted(
                children,
                key=lambda x: (x.__module__, x.__name__),
                )
            for child in children:
                ok_to_join = False
                if self.lineage_prune_distance is None:
                    ok_to_join = True
                elif parent not in distances:
                    if child not in distances:
                        ok_to_join = True
                    elif child in distances and \
                        distances[child] <= ok_distance:
                        ok_to_join = True
                elif child not in distances:
                    if parent not in distances:
                        ok_to_join = True
                    elif parent in distances and \
                        distances[parent] <= ok_distance:
                        ok_to_join = True
                elif distances[child] <= ok_distance and \
                    distances[parent] <= ok_distance:
                    ok_to_join = True
                if ok_to_join:
                    parent_node = class_nodes[parent]
                    child_node = class_nodes[child]
                    graphtools.GraphvizEdge().attach(
                        parent_node, child_node)

        for i, cluster in enumerate(
            sorted(graph.children, key=lambda x: x.name)):
            color = i % 9 + 1
            for node in cluster:
                if 'color' not in node.attributes:
                    node.attributes['color'] = color
                if self.use_groups:
                    node.attributes['group'] = i
            if not self.use_clusters:
                graph.extend(cluster[:])
                graph.remove(cluster)

        if self.root_addresses is None:
            graph.attributes['root'] = '__builtin__.object'

        return graph

    ### PRIVATE METHODS ###

    def _build_basic_mappings(self, classes):
        def recurse(current_class):
            if current_class in child_parents_mapping:
                return True
            elif current_class in invalid_classes:
                return False
            mro = list(inspect.getmro(current_class))
            while len(mro) and mro[-1] not in self.root_classes:
                mro.pop()
            if not mro:
                invalid_classes.add(current_class)
                return False
            parents = [x for x in current_class.__bases__ if recurse(x)]
            child_parents_mapping[current_class] = set(parents)
            parent_children_mapping[current_class] = set([])
            for parent in parents:
                parent_children_mapping[parent].add(current_class)
            return True
        child_parents_mapping = {}
        parent_children_mapping = {}
        invalid_classes = set([])
        for current_class in classes:
            recurse(current_class)
        return child_parents_mapping, parent_children_mapping

    @classmethod
    def _collect_classes(cls, addresses, recurse_into_submodules):
        all_classes = set([])
        cached_addresses = []
        immediate_classes = set([])
        visited_modules = set([])
        assert 0 < len(addresses)
        for x in addresses:
            address = None
            if isinstance(x, (str, types.ModuleType)):
                if isinstance(x, types.ModuleType):
                    module = x
                else:
                    try:
                        module = importlib.import_module(x)
                    except ImportError:
                        module = None
                if module is None:
                    continue
                for y in module.__dict__.values():
                    if isinstance(y, type):
                        all_classes.add(y)
                        immediate_classes.add(y)
                    elif isinstance(y, types.ModuleType) and \
                        recurse_into_submodules:
                        all_classes.update(
                            cls._submodule_recurse(y, visited_modules))
                address = module.__name__
            else:
                if isinstance(x, type):
                    current_class = x
                elif isinstance(x, tuple) and len(x) == 2:
                    module_name, class_name = x
                    module = importlib.import_module(module_name)
                    current_class = getattr(module, class_name)
                else:
                    current_class = x.__class__
                all_classes.add(current_class)
                immediate_classes.add(current_class)
                address = (current_class.__module__, current_class.__name__)
            if address is not None:
                cached_addresses.append(address)
        return all_classes, immediate_classes, tuple(cached_addresses)

    def _find_lineage_distances(self):
        def recurse_downward(current_class, distance=0):
            if current_class not in self.parent_children_mapping:
                return
            for child in self.parent_children_mapping[current_class]:
                if child not in distance_mapping:
                    distance_mapping[child] = distance + 1
                    recurse_downward(child, distance + 1)
                elif (distance + 1) < distance_mapping[child]:
                    distance_mapping[child] = distance + 1
                    recurse_downward(child, distance + 1)
        if not self.lineage_classes:
            return None
        if not self.lineage_prune_distance:
            return None
        distance_mapping = {}
        for current_class in self.lineage_classes:
            recurse_downward(current_class)
        return distance_mapping

    @staticmethod
    def _get_class_name_pieces(current_class):
        parts = (
            current_class.__module__ + '.' + current_class.__name__
            ).split('.')
        name = [parts[0]]
        for part in parts[1:]:
            if part != name[-1]:
                name.append(part)
        if name[0] in ('abjad', 'experimental', 'ide'):
            return name[2:]
        elif 2 < len(name) and name[1] == 'tools':
            return name[2:]
        return name

    @staticmethod
    def _recurse_downward(
        current_class,
        invalid_classes,
        parent_children_mapping,
        ):
        if current_class not in parent_children_mapping:
            return
        for child in parent_children_mapping[current_class]:
            if child in invalid_classes:
                invalid_classes.remove(child)
                InheritanceGraph._recurse_downward(
                    child,
                    invalid_classes,
                    parent_children_mapping,
                    )

    @staticmethod
    def _recurse_upward(
        current_class,
        invalid_classes,
        child_parents_mapping,
        ):
        if current_class not in child_parents_mapping:
            return
        for parent in child_parents_mapping[current_class]:
            if parent in invalid_classes:
                invalid_classes.remove(parent)
                InheritanceGraph._recurse_upward(
                    parent,
                    invalid_classes,
                    child_parents_mapping,
                    )

    def _strip_nonlineage_classes(self,
        child_parents_mapping, parent_children_mapping):
        if not self.lineage_classes:
            return
        invalid_classes = set(
            list(child_parents_mapping.keys()) +
            list(parent_children_mapping.keys())
            )
        for current_class in self.lineage_classes:
            if current_class in invalid_classes:
                invalid_classes.remove(current_class)
            InheritanceGraph._recurse_upward(
                current_class,
                invalid_classes,
                child_parents_mapping,
                )
            InheritanceGraph._recurse_downward(
                current_class,
                invalid_classes,
                parent_children_mapping
                )
        for current_class in invalid_classes:
            for child in parent_children_mapping[current_class]:
                child_parents_mapping[child].remove(current_class)
            for parent in child_parents_mapping[current_class]:
                parent_children_mapping[parent].remove(current_class)
            del(parent_children_mapping[current_class])
            del(child_parents_mapping[current_class])

    @classmethod
    def _submodule_recurse(cls, module, visited_modules):
        result = []
        for obj in list(module.__dict__.values()):
            if isinstance(obj, type):
                result.append(obj)
            elif isinstance(obj, types.ModuleType) and \
                obj not in visited_modules:
                visited_modules.add(obj)
                result.extend(cls._submodule_recurse(obj, visited_modules))
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def addresses(self):
        r'''Addresses of inheritance graph.
        '''
        return self._addresses

    @property
    def child_parents_mapping(self):
        r'''Child / parent mapping of inheritance graph.
        '''
        return self._child_parents_mapping

    @property
    def immediate_classes(self):
        r'''Immediate classes of inheritance graph.
        '''
        return self._immediate_classes

    @property
    def lineage_addresses(self):
        r'''Lineage addresses of inheritance graph.
        '''
        return self._lineage_addresses

    @property
    def lineage_classes(self):
        r'''Lineage classes of inheritance graph.
        '''
        return self._lineage_classes

    @property
    def lineage_distance_mapping(self):
        r'''Lineage distance mapping of inheritance graph.
        '''
        return self._lineage_distance_mapping

    @property
    def lineage_prune_distance(self):
        r'''Lineage prune distance of inheritance graph.
        '''
        return self._lineage_prune_distance

    @property
    def parent_children_mapping(self):
        r'''Parent / children mapping of inheritancee graph.
        '''
        return self._parent_children_mapping

    @property
    def recurse_into_submodules(self):
        r'''Recurse into submodules.
        '''
        return self._recurse_into_submodules

    @property
    def root_addresses(self):
        r'''Root addresses of inheritance graph.
        '''
        return self._root_addresses

    @property
    def root_classes(self):
        r'''Root classes of inheritance graph.
        '''
        return self._root_classes

    @property
    def use_clusters(self):
        r'''Use clusters.
        '''
        return self._use_clusters

    @property
    def use_groups(self):
        r'''Use groups.
        '''
        return self._use_groups



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedInterval___eq___01():

    named_interval_1 = pitchtools.NamedInterval('minor', 2)
    named_interval_2 = pitchtools.NamedInterval('minor', 2)

    assert named_interval_1 == named_interval_2



def test_pitchtools_NamedInterval___eq___02():

    named_interval_1 = pitchtools.NamedInterval('minor', 2)
    named_interval_2 = pitchtools.NamedInterval('augmented', 1)

    assert not named_interval_1 == named_interval_2



# -*- coding: utf-8 -*-
import math


def least_multiple_greater_equal(m, n):
    r'''Gets least integer multiple of `m` greater than or equal to `n`.

    ..  container:: example

        ::

            >>> mathtools.least_multiple_greater_equal(10, 47)
            50

        ::

            >>> for m in range(1, 10):
            ...     print(m, mathtools.least_multiple_greater_equal(m, 47))
            ... 
            1 47
            2 48
            3 48
            4 48
            5 50
            6 48
            7 49
            8 48
            9 54

        ::

            >>> for n in range(10, 100, 10):
            ...     print(mathtools.least_multiple_greater_equal(7, n), n)
            ... 
            14 10
            21 20
            35 30
            42 40
            56 50
            63 60
            70 70
            84 80
            91 90

    Returns integer.
    '''
    result = m * int(math.ceil(n / float(m)))
    return result



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_VerticalMoment_leaves_01():

    score = Score(
        r'''
        \new Staff {
            \times 4/3 {
                d''8
                c''8
                b'8
            }
        }
        \new PianoStaff <<
            \new Staff {
                a'4
                g'4
            }
            \new Staff {
                \clef "bass"
                f'8
                e'8
                d'8
                c'8
            }
        >>
        '''
        )

    staff_group = score[1]

    vertical_moment = inspect_(score).get_vertical_moment_at(Offset(1, 8))

    "(Note(d'', 8), Note(a', 4), Note(e', 8))"

    assert vertical_moment.leaves == (
        score[0][0][0], staff_group[0][0], staff_group[1][1])



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedPitchClass_apply_accidental_01():

    pc = pitchtools.NumberedPitchClass(11)

    assert pc.apply_accidental('sharp') == pitchtools.NumberedPitchClass(0)
    assert pc.apply_accidental('flat') == pitchtools.NumberedPitchClass(10)
    assert pc.apply_accidental('double sharp') == pitchtools.NumberedPitchClass(1)
    assert pc.apply_accidental('double flat') == pitchtools.NumberedPitchClass(9)
    assert pc.apply_accidental('quarter sharp') == pitchtools.NumberedPitchClass(11.5)
    assert pc.apply_accidental('quarter flat') == pitchtools.NumberedPitchClass(10.5)



# -*- coding: utf-8 -*-
import random
from abjad.demos.mozart.make_mozart_measure_corpus \
    import make_mozart_measure_corpus


def choose_mozart_measures():
    r'''Chooses Mozart measures.
    '''

    measure_corpus = make_mozart_measure_corpus()
    chosen_measures = []
    for i, choices in enumerate(measure_corpus):
        if i == 7: # get both alternative endings for mm. 8
            chosen_measures.extend(choices)
        else:
            choice = random.choice(choices)
            chosen_measures.append(choice)
    return chosen_measures



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.quantizationtools import *


def test_quantizationtools_QGrid___call___01():

    q_grid = QGrid()
    a = QEventProxy(SilentQEvent(0,        ['A']), 0)
    b = QEventProxy(SilentQEvent((1, 20),  ['B']), (1, 20))
    c = QEventProxy(SilentQEvent((9, 20),  ['C']), (9, 20))
    d = QEventProxy(SilentQEvent((1, 2),   ['D']), (1, 2))
    e = QEventProxy(SilentQEvent((11, 20), ['E']), (11, 20))
    f = QEventProxy(SilentQEvent((19, 20), ['F']), (19, 20))
    g = QEventProxy(SilentQEvent(1,        ['G']), 1)
    q_grid.fit_q_events([a, b, c, d, e, f, g])
    result = q_grid((1, 4))

    assert len(result) == 1
    assert format(result[0]) == "c'4"

    annotation = inspect_(result[0]).get_indicator(indicatortools.Annotation)

    assert isinstance(annotation.value, tuple) and len(annotation.value) == 4
    assert annotation.value[0].attachments == ('A',)
    assert annotation.value[1].attachments == ('B',)
    assert annotation.value[2].attachments == ('C',)
    assert annotation.value[3].attachments == ('D',)


def test_quantizationtools_QGrid___call___02():

    q_grid = QGrid()
    q_grid.subdivide_leaves([(0, (1, 1, 1))])
    q_grid.subdivide_leaves([(1, (1, 1))])
    q_grid.subdivide_leaves([(-2, (1, 1, 1))])
    a = QEventProxy(SilentQEvent(0,        ['A']), 0)
    b = QEventProxy(SilentQEvent((1, 20),  ['B']), (1, 20))
    c = QEventProxy(SilentQEvent((9, 20),  ['C']), (9, 20))
    d = QEventProxy(SilentQEvent((1, 2),   ['D']), (1, 2))
    e = QEventProxy(SilentQEvent((11, 20), ['E']), (11, 20))
    f = QEventProxy(SilentQEvent((19, 20), ['F']), (19, 20))
    g = QEventProxy(SilentQEvent(1,        ['G']), 1)
    q_grid.fit_q_events([a, b, c, d, e, f, g])
    result = q_grid((1, 4))

    assert isinstance(result, list) and len(result) == 1
    assert format(result[0]) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'8
            c'16
            c'16
            \times 2/3 {
                c'16
                c'16
                c'16
            }
        }
        '''
        )

    leaves = select(result[0]).by_leaf()
    leaf = leaves[0]
    annotation = inspect_(leaf).get_indicators(indicatortools.Annotation)[0]
    assert isinstance(annotation.value, tuple) and len(annotation.value) == 2
    assert annotation.value[0].attachments == ('A',)
    assert annotation.value[1].attachments == ('B',)

    leaf = leaves[1]
    assert not inspect_(leaf).get_indicators(indicatortools.Annotation)

    leaf = leaves[2]
    annotation = inspect_(leaf).get_indicator(indicatortools.Annotation)

    assert isinstance(annotation.value, tuple) and len(annotation.value) == 3
    assert annotation.value[0].attachments == ('C',)
    assert annotation.value[1].attachments == ('D',)
    assert annotation.value[2].attachments == ('E',)

    for leaf in leaves[3:6]:
        assert not inspect_(leaf).get_indicators(indicatortools.Annotation)


def test_quantizationtools_QGrid___call___03():
    r'''Non-binary works too.
    '''

    q_grid = QGrid()
    q_grid.subdivide_leaves([(0, (1, 1))])

    a = QEventProxy(SilentQEvent(0,        ['A']), 0)
    b = QEventProxy(SilentQEvent((1, 20),  ['B']), (1, 20))
    c = QEventProxy(SilentQEvent((9, 20),  ['C']), (9, 20))
    d = QEventProxy(SilentQEvent((1, 2),   ['D']), (1, 2))
    e = QEventProxy(SilentQEvent((11, 20), ['E']), (11, 20))
    f = QEventProxy(SilentQEvent((19, 20), ['F']), (19, 20))
    g = QEventProxy(SilentQEvent(1,        ['G']), 1)

    q_grid.fit_q_events([a, b, c, d, e, f, g])

    result = q_grid((1, 3))

    assert isinstance(result, list) and len(result) == 1
    assert format(result[0]) == stringtools.normalize(
        r'''
        \tweak edge-height #'(0.7 . 0)
        \times 2/3 {
            c'4
            c'4
        }
        '''
        )



# -*- coding: utf-8 -*-


def weight(argument):
    r'''Gets weight of `argument`.

    ..  container:: example

        ::

            >>> mathtools.weight([-1, -2, 3, 4, 5])
            15

    ..  container:: example

        ::

            >>> mathtools.weight([])
            0

    Defined equal to sum of the absolute value of items in `argument`.

    Returns nonnegative integer.
    '''
    return sum([abs(_) for _ in argument])



# -*- coding: utf-8 -*-
from abjad.demos.part.make_part_lilypond_file import make_part_lilypond_file


def test_demos_part_01():

    lilypond_file = make_part_lilypond_file()
    assert len(format(lilypond_file))



# -*- coding: utf-8 -*-
from abjad.tools.topleveltools import select


def get_measure(argument, measure_number):
    r'''Gets measure `measure_number` in `argument`.

    ::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }

    ::

        >>> scoretools.get_measure(staff, 3)
        Measure((2, 8), "g'8 a'8")

    Note that measures number from ``1``.
    '''
    from abjad.tools import scoretools

    # check input
    if measure_number < 1:
        message = 'must be positive integer: {!r}.'
        message = message.format(measure_number)
        raise ValueError(message)

    # calculate measure index
    measure_index = measure_number - 1

    # return measure
    selection = select(argument)
    return selection._get_component(scoretools.Measure, measure_index)



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools.topleveltools import attach


def apply_dynamics(score):
    r'''Applies dynamics to score.
    '''

    voice = score['Bell Voice']
    dynamic = indicatortools.Dynamic('ppp')
    attach(dynamic, voice[0][1])
    dynamic = indicatortools.Dynamic('pp')
    attach(dynamic, voice[8][1])
    dynamic = indicatortools.Dynamic('p')
    attach(dynamic, voice[18][1])
    dynamic = indicatortools.Dynamic('mp')
    attach(dynamic, voice[26][1])
    dynamic = indicatortools.Dynamic('mf')
    attach(dynamic, voice[34][1])
    dynamic = indicatortools.Dynamic('f')
    attach(dynamic, voice[42][1])
    dynamic = indicatortools.Dynamic('ff')
    attach(dynamic, voice[52][1])
    dynamic = indicatortools.Dynamic('fff')
    attach(dynamic, voice[60][1])
    dynamic = indicatortools.Dynamic('ff')
    attach(dynamic, voice[68][1])
    dynamic = indicatortools.Dynamic('f')
    attach(dynamic, voice[76][1])
    dynamic = indicatortools.Dynamic('mf')
    attach(dynamic, voice[84][1])
    dynamic = indicatortools.Dynamic('pp')
    attach(dynamic, voice[-1][0])

    voice = score['First Violin Voice']
    dynamic = indicatortools.Dynamic('ppp')
    attach(dynamic, voice[6][1])
    dynamic = indicatortools.Dynamic('pp')
    attach(dynamic, voice[15][0])
    dynamic = indicatortools.Dynamic('p')
    attach(dynamic, voice[22][3])
    dynamic = indicatortools.Dynamic('mp')
    attach(dynamic, voice[31][0])
    dynamic = indicatortools.Dynamic('mf')
    attach(dynamic, voice[38][3])
    dynamic = indicatortools.Dynamic('f')
    attach(dynamic, voice[47][0])
    dynamic = indicatortools.Dynamic('ff')
    attach(dynamic, voice[55][2])
    dynamic = indicatortools.Dynamic('fff')
    attach(dynamic, voice[62][2])

    voice = score['Second Violin Voice']
    dynamic = indicatortools.Dynamic('pp')
    attach(dynamic, voice[7][0])
    dynamic = indicatortools.Dynamic('p')
    attach(dynamic, voice[12][0])
    dynamic = indicatortools.Dynamic('p')
    attach(dynamic, voice[16][0])
    dynamic = indicatortools.Dynamic('mp')
    attach(dynamic, voice[25][1])
    dynamic = indicatortools.Dynamic('mf')
    attach(dynamic, voice[34][1])
    dynamic = indicatortools.Dynamic('f')
    attach(dynamic, voice[44][1])
    dynamic = indicatortools.Dynamic('ff')
    attach(dynamic, voice[54][0])
    dynamic = indicatortools.Dynamic('fff')
    attach(dynamic, voice[62][1])

    voice = score['Viola Voice']
    dynamic = indicatortools.Dynamic('p')
    attach(dynamic, voice[8][0])
    dynamic = indicatortools.Dynamic('mp')
    attach(dynamic, voice[19][1])
    dynamic = indicatortools.Dynamic('mf')
    attach(dynamic, voice[30][0])
    dynamic = indicatortools.Dynamic('f')
    attach(dynamic, voice[36][0])
    dynamic = indicatortools.Dynamic('f')
    attach(dynamic, voice[42][0])
    dynamic = indicatortools.Dynamic('ff')
    attach(dynamic, voice[52][0])
    dynamic = indicatortools.Dynamic('fff')
    attach(dynamic, voice[62][0])

    voice = score['Cello Voice']
    dynamic = indicatortools.Dynamic('p')
    attach(dynamic, voice[10][0])
    dynamic = indicatortools.Dynamic('mp')
    attach(dynamic, voice[21][0])
    dynamic = indicatortools.Dynamic('mf')
    attach(dynamic, voice[31][0])
    dynamic = indicatortools.Dynamic('f')
    attach(dynamic, voice[43][0])
    dynamic = indicatortools.Dynamic('ff')
    attach(dynamic, voice[52][1])
    dynamic = indicatortools.Dynamic('fff')
    attach(dynamic, voice[62][0])

    voice = score['Bass Voice']
    dynamic = indicatortools.Dynamic('mp')
    attach(dynamic, voice[14][0])
    dynamic = indicatortools.Dynamic('mf')
    attach(dynamic, voice[27][0])
    dynamic = indicatortools.Dynamic('f')
    attach(dynamic, voice[39][0])
    dynamic = indicatortools.Dynamic('ff')
    attach(dynamic, voice[51][0])
    dynamic = indicatortools.Dynamic('fff')
    attach(dynamic, voice[62][0])


# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedIntervalClass___init___01():

    mcic = pitchtools.NumberedIntervalClass(3)

    assert repr(mcic) == 'NumberedIntervalClass(3)'
    assert str(mcic) == '+3'
    assert mcic.number == 3


def test_pitchtools_NumberedIntervalClass___init___02():
    r'''Works with numbers less or equal to -12.
    '''

    mcic = pitchtools.NumberedIntervalClass(-12)
    assert repr(mcic) == 'NumberedIntervalClass(-12)'
    assert str(mcic) == '-12'
    assert mcic.number == -12

    mcic = pitchtools.NumberedIntervalClass(-13)
    assert repr(mcic) == 'NumberedIntervalClass(-1)'
    assert str(mcic) == '-1'
    assert mcic.number == -1


def test_pitchtools_NumberedIntervalClass___init___03():
    r'''Works with numbers greater than 12.
    '''

    mcic = pitchtools.NumberedIntervalClass(12)
    assert repr(mcic) == 'NumberedIntervalClass(12)'
    assert str(mcic) == '+12'
    assert mcic.number == 12

    mcic = pitchtools.NumberedIntervalClass(13)
    assert repr(mcic) == 'NumberedIntervalClass(1)'
    assert str(mcic) == '+1'
    assert mcic.number == 1


def test_pitchtools_NumberedIntervalClass___init___04():
    r'''Works with other interval-class instances.
    '''

    mcic = pitchtools.NumberedIntervalClass(12)
    new_mcic = pitchtools.NumberedIntervalClass(mcic)
    assert repr(mcic) == 'NumberedIntervalClass(12)'
    assert str(mcic) == '+12'
    assert mcic.number == 12



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Measure___setattr___01():
    r'''Slots constraint measure attributes.
    '''

    measure = Measure((3, 8), "c'8 d'8 e'8")

    assert pytest.raises(AttributeError, "measure.foo = 'bar'")



# -*- coding: utf-8 -*-


def select_every(indices, period=None, inverted=None):
    r'''Makes pattern that matches `indices` at `period`.

    ..  container:: example

        Selects every second division:

        ::

            >>> mask = patterntools.select_every(indices=[1], period=2)

        ::

            >>> print(format(mask))
            patterntools.Pattern(
                indices=[1],
                period=2,
                )

        ::

            >>> maker = rhythmmakertools.NoteRhythmMaker(
            ...     division_masks=[mask],
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    r4.
                }
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    r4.
                }
            }

    ..  container:: example

        Selects every second and third division:

        ::

            >>> mask = patterntools.select_every(indices=[1, 2], period=3)

        ::

            >>> print(format(mask))
            patterntools.Pattern(
                indices=[1, 2],
                period=3,
                )

        ::

            >>> maker = rhythmmakertools.NoteRhythmMaker(
            ...     division_masks=[mask],
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    r4.
                }
                {
                    \time 7/16
                    r4..
                }
                {
                    \time 3/8
                    c'4.
                }
            }


    Returns pattern.
    '''
    from abjad.tools import patterntools
    return patterntools.Pattern(
        indices=indices,
        inverted=inverted,
        period=period,
        )



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.markuptools import Markup
from abjad.tools.pitchtools import NamedPitch
from abjad.tools.pitchtools import PitchRange


def test_pitchtools_PitchRange___repr___01():
    r'''Pitch range reprs are evaluable.
    '''

    pitch_range_1 = pitchtools.PitchRange.from_pitches(-12, 36)
    pitch_range_2 = eval(repr(pitch_range_1))

    assert isinstance(pitch_range_1, PitchRange)
    assert isinstance(pitch_range_2, PitchRange)

    assert pitch_range_1 == pitch_range_2
    assert not pitch_range_1 is pitch_range_2


def test_pitchtools_PitchRange___repr___02():

    pitch_range = pitchtools.PitchRange.from_pitches(-12, 36)
    assert repr(pitch_range) == "PitchRange(range_string='[C3, C7]')"



# -*- coding: utf-8 -*-
from abjad.tools import systemtools
from abjad.tools.schemetools.Scheme import Scheme


class SchemeSymbol(Scheme):
    r'''A Scheme symbol.

    ..  container:: example

        ::

            >>> scheme = schemetools.SchemeSymbol('cross')
            >>> scheme
            SchemeSymbol('cross')

        ::

            >>> print(format(scheme))
            #'cross

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, symbol=None):
        if symbol is None:
            symbol = 'cross'
        symbol = str(symbol)
        Scheme.__init__(
            self,
            symbol,
            quoting="'",
            )

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = [self._value]
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=values,
            storage_format_kwargs_names=[],
            )

    ### PUBLIC PROPERTIES ###

    @property
    def symbol(self):
        r'''Gets symbol string.

        Returns string.
        '''
        return self._value



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Tuplet_from_leaf_and_ratio_01():

    note = Note(0, (3, 16))

    tuplet = Tuplet.from_leaf_and_ratio(note, [1], is_diminution=False)

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8.
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(note, [1, 2], is_diminution=False)

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'8
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        [1, 2, 2, 3],
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/2 {
            c'64
            c'32
            c'32
            c'32.
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        [1, 2, 2, 3, 3],
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 12/11 {
            c'64
            c'32
            c'32
            c'32.
            c'32.
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        [1, 2, 2, 3, 3, 4],
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 8/5 {
            c'128
            c'64
            c'64
            c'64.
            c'64.
            c'32
        }
        '''
        )


def test_scoretools_Tuplet_from_leaf_and_ratio_02():

    note = Note("c'8.")

    tuplet = Tuplet.from_leaf_and_ratio(note, [1], is_diminution=True)

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8.
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(note, [1, 2], is_diminution=True)

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'8
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(note, [1, 2, 2, 3], is_diminution=True)

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/4 {
            c'32
            c'16
            c'16
            c'16.
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        [1, 2, 2, 3, 3],
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 6/11 {
            c'32
            c'16
            c'16
            c'16.
            c'16.
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        [1, 2, 2, 3, 3, 4],
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 4/5 {
            c'64
            c'32
            c'32
            c'32.
            c'32.
            c'16
        }
        '''
        )


def test_scoretools_Tuplet_from_leaf_and_ratio_03():
    r'''Divide note into 1, ..., 5 parts.
    '''

    note = Note("c'8.")

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        1 * [1],
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8.
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        2 * [1],
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16.
            c'16.
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        3 * [1],
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'16
            c'16
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        4 * [1],
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'32.
            c'32.
            c'32.
            c'32.
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        5 * [1],
        is_diminution=False,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 8/5 {
            c'64.
            c'64.
            c'64.
            c'64.
            c'64.
        }
        '''
        )


def test_scoretools_Tuplet_from_leaf_and_ratio_04():
    r'''Divide note into 1, ..., 5 parts.
    '''

    note = Note("c'8.")

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        1 * [1],
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'8.
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        2 * [1],
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16.
            c'16.
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        3 * [1],
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'16
            c'16
            c'16
        }
        '''
        )

    tuplet = Tuplet.from_leaf_and_ratio(
        note,
        4 * [1],
        is_diminution=True,
        )

    assert format(tuplet) == stringtools.normalize(
        r'''
        {
            c'32.
            c'32.
            c'32.
            c'32.
        }
        '''
        )



# -*- coding: utf-8 -*-
import numbers
from abjad.tools import mathtools
from abjad.tools.pitchtools.NumberedIntervalClass import NumberedIntervalClass


class NumberedInversionEquivalentIntervalClass(NumberedIntervalClass):
    '''Numbered inversion-equivalent interval-class.

    ..  container:: example

        Initializes from integer:

        ::

            >>> pitchtools.NumberedInversionEquivalentIntervalClass(1)
            NumberedInversionEquivalentIntervalClass(1)

    ..  container:: example

        Initializes from string:

        ::

            >>> pitchtools.NumberedInversionEquivalentIntervalClass('1')
            NumberedInversionEquivalentIntervalClass(1)

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        )

    ### INITIALIZER ###

    def __init__(self, interval_class_token=None):
        from abjad.tools import pitchtools
        if isinstance(interval_class_token, type(self)):
            number = interval_class_token.number
        elif isinstance(interval_class_token, numbers.Number):
            if not 0 <= interval_class_token <= 6:
                message = 'must be between 0 and 6, inclusive.'
                raise ValueError(message)
            number = interval_class_token
        elif hasattr(interval_class_token, 'semitones'):
            number = interval_class_token.semitones
            number %= 12
            if 6 < number:
                number = 12 - number
        elif interval_class_token is None:
            number = 0
        elif isinstance(interval_class_token, str):
            number = float(interval_class_token)
            if mathtools.is_integer_equivalent(number):
                number = int(number)
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, interval_class_token)
            raise TypeError(message)
        self._number = number

    ### SPECIAL METHODS ###

    def __abs__(self):
        r'''Absolute value of numbered inversion-equivalent interval-class.

        Returns new numbered inversion-equivalent interval-class.
        '''
        return type(self)(abs(self.number))

    def __copy__(self):
        r'''Copies numbered inversion-equivalent interval-class.

        Returns new numbered inversion-equivalent interval-class.
        '''
        return type(self)(self.number)

    def __lt__(self, argument):
        r'''Is true when `argument` is a numbered inversion-equivalent
        interval-class with a number less than this numbered
        inversion-equivalent interval-class.
        '''
        if isinstance(argument, type(self)):
            return self.number < argument.number
        return False

    def __neg__(self):
        r'''Negates numbered inversion-equivalent interval-class.

        Returns new numbered inversion-equivalent interval-class.
        '''
        return type(self)(self.number)

    def __str__(self):
        r'''String representation of numbered inversion-equivalent
        interval-class.

        Returns string.
        '''
        return str(self.number)



# -*- coding: utf-8 -*-
from __future__ import print_function
import os
import sys
import traceback
from abjad.tools import stringtools
from abjad.tools import systemtools
from abjad.tools.commandlinetools.ScorePackageScript import ScorePackageScript


class ManageSegmentScript(ScorePackageScript):
    '''
    Manages score package segments.

    ..  shell::

        ajv segment --help

    '''

    ### CLASS VARIABLES ###

    alias = 'segment'
    short_description = 'Manage score package segments.'

    ### PRIVATE METHODS ###

    def _collect_matching_paths(self, names, include_unstaged):
        valid_paths = self._list_segment_subpackages()
        staged_names = self._read_segments_list_json()
        staged_matching_paths, unstaged_matching_paths = [], []
        for name in names:
            if name.startswith('+'):
                name = '*{}*'.format('*'.join(name[1:]))
            for path in sorted(self._segments_path.glob(name)):
                if path not in valid_paths:
                    continue
                elif (
                    path in staged_matching_paths or
                    path in unstaged_matching_paths
                    ):
                    continue
                if path.name not in staged_names:
                    if include_unstaged:
                        unstaged_matching_paths.append(path)
                else:
                    staged_matching_paths.append(path)
        matching_paths = []
        matching_paths.extend(sorted(staged_matching_paths,
            key=lambda path: staged_names.index(path.name)))
        matching_paths.extend(sorted(unstaged_matching_paths))
        return matching_paths

    def _create_staging_contents(self, all_names, staged_names):
        names_list = list(staged_names)
        for name in all_names:
            if name not in staged_names:
                names_list.append('# {}'.format(name))
        names_list = '\n'.join(names_list)
        contents = stringtools.normalize('''
        {}

        # Instructions:
        #
        # Reorder, comment or uncomment the segment names
        # above to stage those segments for illustration.
        #
        # Lines beginning with "#" will be unstaged.
        # Duplicates will be ignored.
        # Unknown names will be ignored.
        ''').format(names_list)
        return contents

    def _handle_collect(self):
        print('Collecting segments:')
        collected_names = []
        for segment_path in self._list_segment_subpackages(
            self._score_package_path):
            source_path = segment_path.joinpath('illustration.ly')
            if not source_path.is_file():
                continue
            collected_names.append(segment_path.name)
            target_path = self._build_path.joinpath(
                'segments',
                stringtools.to_dash_case(segment_path.name) + '.ily'
                )
            contents = self._process_illustration_contents(source_path)
            with open(str(target_path), 'w') as file_pointer:
                file_pointer.write(contents)
            message = '    {!s} --> {!s}'.format(
                source_path.relative_to(self._score_package_path),
                target_path.relative_to(self._score_package_path),
                )
            print(message)
        staged_names = self._read_segments_list_json(self._score_package_path)
        segments_ily_path = self._build_path.joinpath('segments.ily')
        include_template = '    \\include "..{sep}segments{sep}{name}.ily"\n'
        with open(str(segments_ily_path), 'w') as file_pointer:
            file_pointer.write('{\n')
            for name in staged_names:
                if name not in collected_names:
                    continue
                name = stringtools.to_dash_case(name)
                file_pointer.write(include_template.format(
                    name=name,
                    sep=os.path.sep))
            file_pointer.write('}\n')

    def _handle_create(self, segment_name, force):
        print('Creating segment subpackage {!r} ...'.format(segment_name))
        segment_path = self._name_to_score_subdirectory_path(
            segment_name, 'segments', self._score_package_path)
        if segment_path.exists() and not force:
            print('    Path exists: {}'.format(
                segment_path.relative_to(self._score_package_path.parent)))
            sys.exit(1)
        segment_name = segment_path.name
        metadata = self._read_score_metadata_json()
        metadata['score_package_name'] = self._score_package_path.name
        source_name = 'example_segment'
        source_path = self._get_boilerplate_path().joinpath(source_name)
        suffixes = ('.py', '.tex', '.ly', '.ily')
        for path in self._copy_tree(source_path, segment_path):
            if path.is_file() and path.suffix in suffixes:
                self._template_file(path, **metadata)
        segment_names = self._read_segments_list_json()
        if segment_name not in segment_names:
            segment_names.append(segment_name)
            self._write_segments_list_json(segment_names)
        print('    Created {path!s}{sep}'.format(
            path=segment_path.relative_to(self._score_package_path.parent),
            sep=os.path.sep,
            ))

    def _handle_edit(self, segment_name):
        from abjad import abjad_configuration
        segment_name = segment_name or '*'
        globbable_names = self._collect_globbable_names(segment_name)
        print('Edit candidates: {!r} ...'.format(
            ' '.join(globbable_names)))
        matching_paths = self._collect_matching_paths(
            globbable_names,
            include_unstaged=True,
            )
        if not matching_paths:
            print('    No matching segments.')
            self._handle_list()
        command = [abjad_configuration.get_text_editor()]
        for path in matching_paths:
            command.append(str(path.joinpath('definition.py')))
        command = ' '.join(command)
        exit_code = self._call_subprocess(command)
        if exit_code:
            sys.exit(exit_code)

    def _handle_illustrate(self, segment_name, unstaged=False):
        globbable_names = self._collect_globbable_names(segment_name)
        print('Illustration candidates: {!r} ...'.format(
            ' '.join(globbable_names)))
        matching_paths = self._collect_matching_paths(
            globbable_names,
            include_unstaged=unstaged,
            )
        if not matching_paths:
            print('    No matching segments.')
            self._handle_list()
        for path in matching_paths:
            self._illustrate_one_segment(segment_directory_path=path)
            print('    Illustrated {path!s}{sep}'.format(
                path=path.relative_to(self._score_package_path.parent),
                sep=os.path.sep))
        for path in matching_paths:
            pdf_path = path.joinpath('illustration.pdf')
            systemtools.IOManager.open_file(str(pdf_path))

    def _handle_list(self):
        print('Available segments:')
        valid_paths = self._list_segment_subpackages(self._score_package_path)
        valid_names = [_.name for _ in valid_paths]
        staged_names = [_ for _ in self._read_segments_list_json()
            if _ in valid_names]
        unstaged_names = [_ for _ in valid_names if _ not in staged_names]
        if staged_names:
            max_length = max(len(_) for _ in staged_names)
            for i, name in enumerate(staged_names, 1):
                spacing = ' ' * (max_length - len(name))
                print('    {}{} [{}]'.format(name, spacing, i))
        for name in unstaged_names:
            print('    {}'.format(name))
        if not staged_names and not unstaged_names:
            print('    No segments available.')
        sys.exit(2)

    def _handle_render(self, segment_name, unstaged=False):
        globbable_names = self._collect_globbable_names(segment_name)
        print('Rendering candidates: {!r} ...'.format(
            ' '.join(globbable_names)))
        matching_paths = self._collect_matching_paths(
            globbable_names,
            unstaged,
            )
        if not matching_paths:
            print('    No matching segments.')
            self._handle_list()
        for path in matching_paths:
            self._render_one_segment(segment_directory_path=path)
            print('    Rendered {path!s}{sep}'.format(
                path=path.relative_to(self._score_package_path.parent),
                sep=os.path.sep))
        for path in matching_paths:
            pdf_path = path.joinpath('illustration.pdf')
            systemtools.IOManager.open_file(str(pdf_path))

    def _handle_stage(self):
        from abjad import abjad_configuration
        print('Staging segments:')
        all_names = [
            path.name for path in
            self._list_segment_subpackages(self._score_package_path)
            ]
        old_staged_names = self._read_segments_list_json()
        contents = self._create_staging_contents(all_names, old_staged_names)
        with systemtools.TemporaryDirectory() as directory_path:
            with systemtools.TemporaryDirectoryChange(directory_path):
                file_path = 'segments.txt'
                with open(file_path, 'w') as file_pointer:
                    file_pointer.write(contents)
                command = '{} {}'.format(
                    abjad_configuration.get_text_editor(),
                    file_path,
                    )
                self._call_subprocess(command)
                with open(file_path, 'r') as file_pointer:
                    contents = file_pointer.read()
        lines = (line.strip() for line in contents.splitlines())
        lines = (line for line in lines if not line.startswith('#'))
        lines = (line for line in lines if line in all_names)
        new_staged_names = list(lines)
        if new_staged_names != old_staged_names:
            self._write_segments_list_json(new_staged_names)
        print('Staged:')
        for name in new_staged_names:
            print('    {}'.format(name))

    def _illustrate_one_segment(self, segment_directory_path):
        print('Illustrating {path!s}{sep}'.format(
            path=segment_directory_path.relative_to(self._score_package_path.parent),
            sep=os.path.sep))
        segment_name = segment_directory_path.name
        segment_names = self._read_segments_list_json()
        previous_segment_name = None
        index = None
        if segment_name in segment_names:
            index = segment_names.index(segment_name)
            if 0 < index:
                previous_segment_name = segment_names[index - 1]
            index += 1
        previous_segment_metadata = {}
        if previous_segment_name:
            previous_segment_metadata_path = self._segments_path.joinpath(
                previous_segment_name,
                'metadata.json',
                )
            previous_segment_metadata = self._read_json(
                previous_segment_metadata_path)
        segment_metadata_path = segment_directory_path.joinpath('metadata.json')
        segment_metadata = self._read_json(segment_metadata_path)
        segment_metadata['segment_count'] = len(segment_names)
        segment_metadata['segment_number'] = index
        segment_metadata['first_bar_number'] = (
            previous_segment_metadata.get('measure_count', 0) +
            previous_segment_metadata.get('first_bar_number', 1)
            )
        segment_package_path = self._path_to_packagesystem_path(
            segment_directory_path)
        definition_import_path = segment_package_path + '.definition'
        try:
            module = self._import_path(
                definition_import_path,
                self._score_repository_path,
                )
            segment_maker = getattr(module, 'segment_maker')
        except (ImportError, AttributeError):
            traceback.print_exc()
            sys.exit(1)
        with systemtools.Timer() as timer:
            try:
                lilypond_file, segment_metadata = segment_maker(
                    segment_metadata=segment_metadata,
                    previous_segment_metadata=previous_segment_metadata,
                    )
            except:
                traceback.print_exc()
                sys.exit(1)
            self._write_json(segment_metadata, segment_metadata_path)
        self._report_time(timer, prefix='Abjad runtime')
        ly_path = self._write_lilypond_ly(
            lilypond_file,
            output_directory_path=segment_directory_path,
            )
        self._write_lilypond_pdf(
            ly_path=ly_path,
            output_directory_path=segment_directory_path,
            )

    def _process_args(self, arguments):
        self._setup_paths(arguments.score_path)
        if arguments.collect:
            self._handle_collect()
        if arguments.edit is not None:
            self._handle_edit(segment_name=arguments.edit)
        if arguments.illustrate is not None:
            self._handle_illustrate(
                segment_name=arguments.illustrate,
                unstaged=arguments.unstaged,
                )
        if arguments.list_:
            self._handle_list()
        if arguments.new:
            self._handle_create(force=arguments.force, segment_name=arguments.new)
        if arguments.render is not None:
            self._handle_render(
                segment_name=arguments.render,
                unstaged=arguments.unstaged,
                )
        if arguments.stage:
            self._handle_stage()

    def _process_illustration_contents(self, ly_path):
        with open(str(ly_path), 'r') as file_pointer:
            contents = file_pointer.read().splitlines()
        while contents and contents[0] != r'\score {':
            contents.pop(0)
        contents.pop(0)
        for i in range(len(contents)):
            if contents[i] == '}':
                contents = contents[:i]
                break
        contents = '\n'.join(contents)
        return contents

    def _render_one_segment(self, segment_directory_path):
        print('Rendering {path!s}{sep}'.format(
            path=segment_directory_path.relative_to(self._score_package_path.parent),
            sep=os.path.sep))
        ly_path = segment_directory_path.joinpath('illustration.ly')
        if not ly_path.is_file():
            print('    illustration.ly is missing or malformed.')
            sys.exit(1)
        self._write_lilypond_pdf(
            ly_path=ly_path,
            output_directory_path=segment_directory_path,
            )

    def _setup_argument_parser(self, parser):
        action_group = parser.add_argument_group('actions')
        action_group = action_group.add_mutually_exclusive_group(required=True)
        action_group.add_argument(
            '--new', '-N',
            help='create a new segment',
            metavar='NAME',
            )
        action_group.add_argument(
            '--edit', '-E',
            help='edit segments',
            metavar='PATTERN',
            nargs='*',
            )
        action_group.add_argument(
            '--illustrate', '-I',
            help='illustrate segments',
            metavar='PATTERN',
            nargs='+',
            )
        action_group.add_argument(
            '--render', '-R',
            help='render segment illustrations',
            metavar='PATTERN',
            nargs='+',
            )
        action_group.add_argument(
            '--collect', '-C',
            help='collect segment illustrations',
            action='store_true',
            )
        action_group.add_argument(
            '--stage', '-T',
            help='stage segments for illustration',
            action='store_true',
            )
        action_group.add_argument(
            '--list', '-L',
            dest='list_',
            help='list segments',
            action='store_true',
            )
#        action_group.add_argument(
#            '--copy', '-Y',
#            help='copy segment',
#            metavar=('SOURCE', 'TARGET'),
#            nargs=2,
#            )
#        action_group.add_argument(
#            '--rename', '-M',
#            help='rename segment',
#            metavar=('SOURCE', 'TARGET'),
#            nargs=2,
#            )
#        action_group.add_argument(
#            '--delete', '-D',
#            help='delete segment',
#            metavar='NAME',
#            )
        common_group = parser.add_argument_group('common options')
        common_group.add_argument(
            '--score-path', '-s',
            metavar='SCORE',
            help='score path or package name',
            default=os.path.curdir,
            )
        common_group.add_argument(
            '--force', '-f',
            action='store_true',
            help='force overwriting',
            )
        common_group.add_argument(
            '-u', '--unstaged',
            help='Include segments not staged in segments{sep}metadata.json'.format(sep=os.path.sep),
            action='store_true',
            )



# -*- coding: utf-8 -*-
import functools
from abjad.tools import markuptools
from abjad.tools import mathtools
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.topleveltools.new import new


@functools.total_ordering
class ColorFingering(AbjadValueObject):
    r'''Color fingering.

    ..  container:: example

        **Example 1.** First color fingering:

        ::

            >>> fingering = indicatortools.ColorFingering(1)
            >>> note = Note("c'4")
            >>> attach(fingering, note)

        ::

            >>> show(note) # doctest: +SKIP

        ..  doctest::

            >>> f(note)
            c'4
                ^ \markup {
                    \override
                        #'(circle-padding . 0.25)
                        \circle
                            \finger
                                1
                    }

    ..  container:: example

        **Example 2.** Second color fingering:

        ::

            >>> fingering = indicatortools.ColorFingering(2)
            >>> note = Note("c'4")
            >>> attach(fingering, note)

        ::

            >>> show(note) # doctest: +SKIP

        ..  doctest::

            >>> f(note)
            c'4
                ^ \markup {
                    \override
                        #'(circle-padding . 0.25)
                        \circle
                            \finger
                                2
                    }

    Color fingerings indicate alternate woodwind fingerings by amount of pitch
    of timbre deviation.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_number',
        )

    _format_slot = 'right'

    ### INITIALIZER ###

    def __init__(
        self,
        number=None,
        ):
        self._default_scope = None
        if number is not None:
            assert mathtools.is_positive_integer(number)
        self._number = number

    ### SPECIAL METHODS ##

    def __format__(self, format_specification=''):
        r'''Formats color fingering.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ::

            >>> fingering = indicatortools.ColorFingering(1)
            >>> print(format(fingering))
            indicatortools.ColorFingering(
                number=1,
                )

        Returns string.
        '''
        if format_specification == 'lilypond':
            return self._get_lilypond_format()
        superclass = super(ColorFingering, self)
        return superclass.__format__(format_specification=format_specification)

    def __lt__(self, argument):
        r'''Is true if `argument` is a color fingering and the number of this color
        fingering is less than that of `argument`.

        ..  container:: example

            ::

                >>> fingering_1 = indicatortools.ColorFingering(1)
                >>> fingering_2 = indicatortools.ColorFingering(1)
                >>> fingering_3 = indicatortools.ColorFingering(2)

            ::

                >>> fingering_1 < fingering_1
                False
                >>> fingering_1 < fingering_2
                False
                >>> fingering_1 < fingering_3
                True

            ::

                >>> fingering_2 < fingering_1
                False
                >>> fingering_2 < fingering_2
                False
                >>> fingering_2 < fingering_3
                True

            ::

                >>> fingering_3 < fingering_1
                False
                >>> fingering_3 < fingering_2
                False
                >>> fingering_3 < fingering_3
                False

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return self.number < argument.number
        raise TypeError('unorderable types')

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return repr(self.number)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return format(self.markup, 'lilypond')

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        markup = self.markup
        markup = new(markup, direction=Up)
        markup_format_pieces = markup._get_format_pieces()
        lilypond_format_bundle.right.markup.extend(markup_format_pieces)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of color fingering.

        ..  container:: example

            ::

                >>> fingering = indicatortools.ColorFingering(1)
                >>> fingering.default_scope is None
                True

        Returns none.
        '''
        return self._default_scope

    @property
    def markup(self):
        r'''Gets markup of color fingering.

        ..  container:: example

            **Example 1.** First color fingering:

            ::

                >>> fingering = indicatortools.ColorFingering(1)
                >>> print(format(fingering.markup, 'lilypond'))
                \markup {
                    \override
                        #'(circle-padding . 0.25)
                        \circle
                            \finger
                                1
                    }
                >>> show(fingering.markup) # doctest: +SKIP

        ..  container:: example

            **Example 2.** Second color fingering:

            ::

                >>> fingering = indicatortools.ColorFingering(2)
                >>> print(format(fingering.markup, 'lilypond'))
                \markup {
                    \override
                        #'(circle-padding . 0.25)
                        \circle
                            \finger
                                2
                    }
                >>> show(fingering.markup) # doctest: +SKIP

        Returns markup.
        '''
        if self.number is None:
            return
        markup = markuptools.Markup(str(self.number))
        markup = markup.finger()
        markup = markup.circle()
        markup = markup.override(('circle-padding', 0.25))
        return markup

    @property
    def number(self):
        r'''Gets number of color fingering.

        ..  container:: example

            **Example 1.** First color fingering:

            ::

                >>> fingering = indicatortools.ColorFingering(1)
                >>> fingering.number
                1

        ..  container:: example

            **Example 2.** Second color fingering:

            ::

                >>> fingering = indicatortools.ColorFingering(2)
                >>> fingering.number
                2

        Returns positive integer.
        '''
        return self._number
